@@ -1,4 +1,3 @@
-from collections import OrderedDict
 from copy import copy, deepcopy
 from textwrap import dedent
 try:
@@ -9,10 +8,11 @@ except ImportError:
 import numpy as np
 import pandas as pd
 
-from xray import Dataset, DataArray, Variable, backends, utils, align, indexing
-from xray.pycompat import iteritems
+from xray import (Dataset, DataArray, Coordinate, Variable,
+                  backends, utils, align, indexing)
+from xray.pycompat import iteritems, OrderedDict
 
-from . import TestCase
+from . import TestCase, unittest
 
 
 _dims = {'dim1': 100, 'dim2': 50, 'dim3': 10}
@@ -51,7 +51,7 @@ class InaccessibleArray(utils.NDArrayMixin):
 
 class InaccessibleVariableDataStore(backends.InMemoryDataStore):
     def __init__(self):
-        self.dimensions = OrderedDict()
+        self.dims = OrderedDict()
         self._variables = OrderedDict()
         self.attrs = OrderedDict()
 
@@ -61,7 +61,7 @@ class InaccessibleVariableDataStore(backends.InMemoryDataStore):
 
     def open_store_variable(self, var):
         data = indexing.LazilyIndexedArray(InaccessibleArray(var.values))
-        return Variable(var.dimensions, data, var.attrs)
+        return Variable(var.dims, data, var.attrs)
 
     @property
     def store_variables(self):
@@ -102,7 +102,7 @@ class TestDataset(TestCase):
         actual = '\n'.join(x.rstrip() for x in repr(Dataset()).split('\n'))
         self.assertEqual(expected, actual)
 
-    def test_init(self):
+    def test_constructor(self):
         var1 = Variable('x', 2 * np.arange(100))
         var2 = Variable('x', np.arange(1000))
         var3 = Variable(['x', 'y'], np.arange(1000).reshape(100, 10))
@@ -130,18 +130,18 @@ class TestDataset(TestCase):
         with self.assertRaises(ValueError):
             a['qux'] = (('time', 'x'), d.T)
 
-    def test_coordinate(self):
+    def test_coords_create(self):
         a = Dataset()
         vec = np.random.random((10,))
         attributes = {'foo': 'bar'}
         a['x'] = ('x', vec, attributes)
-        self.assertTrue('x' in a.coordinates)
-        self.assertIsInstance(a.coordinates['x'].as_index, pd.Index)
-        self.assertVariableEqual(a.coordinates['x'], a.variables['x'])
+        self.assertTrue('x' in a.coords)
+        self.assertIsInstance(a.coords['x'].to_index(), pd.Index)
+        self.assertVariableIdentical(a.coords['x'], a.variables['x'])
         b = Dataset()
         b['x'] = ('x', vec, attributes)
-        self.assertVariableEqual(a['x'], b['x'])
-        self.assertEqual(a.dimensions, b.dimensions)
+        self.assertVariableIdentical(a['x'], b['x'])
+        self.assertEqual(a.dims, b.dims)
         # this should work
         a['x'] = ('x', vec[:5])
         a['z'] = ('x', np.arange(5))
@@ -154,7 +154,50 @@ class TestDataset(TestCase):
             a['y'] = ('y', arr)
         with self.assertRaises(ValueError):
             a['y'] = ('y', scal)
-        self.assertTrue('y' not in a.dimensions)
+        self.assertTrue('y' not in a.dims)
+
+    def test_coords_properties(self):
+        data = Dataset({'x': ('x', [-1, -2]),
+                        'y': ('y', [0, 1, 2]),
+                        'foo': (['x', 'y'], np.random.randn(2, 3))})
+
+        self.assertEquals(2, len(data.coords))
+
+        self.assertEquals(set(['x', 'y']), set(data.coords))
+
+        self.assertVariableIdentical(data.coords['x'], data['x'].variable)
+        self.assertVariableIdentical(data.coords['y'], data['y'].variable)
+
+        self.assertIn('x', data.coords)
+        self.assertNotIn(0, data.coords)
+        self.assertNotIn('foo', data.coords)
+
+        with self.assertRaises(KeyError):
+            data.coords['foo']
+        with self.assertRaises(KeyError):
+            data.coords[0]
+
+        expected = dedent("""\
+        x: Int64Index([-1, -2], dtype='int64')
+        y: Int64Index([0, 1, 2], dtype='int64')""")
+        actual = repr(data.coords)
+        self.assertEquals(expected, actual)
+
+    def test_coords_modify(self):
+        data = Dataset({'x': ('x', [-1, -2]),
+                        'y': ('y', [0, 1, 2]),
+                        'foo': (['x', 'y'], np.random.randn(2, 3))})
+
+        actual = data.copy(deep=True)
+        actual.coords['x'] = ['a', 'b']
+        self.assertArrayEqual(actual['x'], ['a', 'b'])
+
+        actual = data.copy(deep=True)
+        actual.coords['z'] = ['a', 'b']
+        self.assertArrayEqual(actual['z'], ['a', 'b'])
+
+        with self.assertRaisesRegexp(ValueError, 'coordinate has size'):
+            data.coords['x'] = [-1]
 
     def test_equals_and_identical(self):
         data = create_test_data(seed=42)
@@ -171,33 +214,37 @@ class TestDataset(TestCase):
         self.assertFalse(data.equals(data2))
         self.assertTrue(data != data2)
 
+        data = create_test_data(seed=42).rename({'var1': None})
+        self.assertTrue(data.equals(data))
+        self.assertTrue(data.identical(data))
+
     def test_attrs(self):
         data = create_test_data(seed=42)
         data.attrs = {'foobar': 'baz'}
         self.assertTrue(data.attrs['foobar'], 'baz')
         self.assertIsInstance(data.attrs, OrderedDict)
 
-    def test_indexed(self):
+    def test_isel(self):
         data = create_test_data()
         slicers = {'dim1': slice(None, None, 2), 'dim2': slice(0, 2)}
-        ret = data.indexed(**slicers)
+        ret = data.isel(**slicers)
 
         # Verify that only the specified dimension was altered
-        self.assertItemsEqual(data.dimensions, ret.dimensions)
-        for d in data.dimensions:
+        self.assertItemsEqual(data.dims, ret.dims)
+        for d in data.dims:
             if d in slicers:
-                self.assertEqual(ret.dimensions[d],
-                                 np.arange(data.dimensions[d])[slicers[d]].size)
+                self.assertEqual(ret.dims[d],
+                                 np.arange(data.dims[d])[slicers[d]].size)
             else:
-                self.assertEqual(data.dimensions[d], ret.dimensions[d])
+                self.assertEqual(data.dims[d], ret.dims[d])
         # Verify that the data is what we expect
         for v in data.variables:
-            self.assertEqual(data[v].dimensions, ret[v].dimensions)
+            self.assertEqual(data[v].dims, ret[v].dims)
             self.assertEqual(data[v].attrs, ret[v].attrs)
             slice_list = [slice(None)] * data[v].values.ndim
             for d, s in iteritems(slicers):
-                if d in data[v].dimensions:
-                    inds = np.nonzero(np.array(data[v].dimensions) == d)[0]
+                if d in data[v].dims:
+                    inds = np.nonzero(np.array(data[v].dims) == d)[0]
                     for ind in inds:
                         slice_list[ind] = s
             expected = data[v].values[slice_list]
@@ -205,24 +252,24 @@ class TestDataset(TestCase):
             np.testing.assert_array_equal(expected, actual)
 
         with self.assertRaises(ValueError):
-            data.indexed(not_a_dim=slice(0, 2))
+            data.isel(not_a_dim=slice(0, 2))
 
-        ret = data.indexed(dim1=0)
-        self.assertEqual({'time': 20, 'dim2': 50, 'dim3': 10}, ret.dimensions)
-        self.assertItemsEqual(list(data.noncoordinates) + ['dim1'],
-                              ret.noncoordinates)
+        ret = data.isel(dim1=0)
+        self.assertEqual({'time': 20, 'dim2': 50, 'dim3': 10}, ret.dims)
+        self.assertItemsEqual(list(data.noncoords) + ['dim1'],
+                              ret.noncoords)
 
-        ret = data.indexed(time=slice(2), dim1=0, dim2=slice(5))
-        self.assertEqual({'time': 2, 'dim2': 5, 'dim3': 10}, ret.dimensions)
-        self.assertItemsEqual(list(data.noncoordinates) + ['dim1'],
-                              ret.noncoordinates)
+        ret = data.isel(time=slice(2), dim1=0, dim2=slice(5))
+        self.assertEqual({'time': 2, 'dim2': 5, 'dim3': 10}, ret.dims)
+        self.assertItemsEqual(list(data.noncoords) + ['dim1'],
+                              ret.noncoords)
 
-        ret = data.indexed(time=0, dim1=0, dim2=slice(5))
-        self.assertItemsEqual({'dim2': 5, 'dim3': 10}, ret.dimensions)
-        self.assertItemsEqual(list(data.noncoordinates) + ['dim1', 'time'],
-                              ret.noncoordinates)
+        ret = data.isel(time=0, dim1=0, dim2=slice(5))
+        self.assertItemsEqual({'dim2': 5, 'dim3': 10}, ret.dims)
+        self.assertItemsEqual(list(data.noncoords) + ['dim1', 'time'],
+                              ret.noncoords)
 
-    def test_labeled(self):
+    def test_sel(self):
         data = create_test_data()
         int_slicers = {'dim1': slice(None, None, 2),
                        'dim2': slice(2),
@@ -230,22 +277,26 @@ class TestDataset(TestCase):
         loc_slicers = {'dim1': slice(None, None, 2),
                        'dim2': slice(0, 0.5),
                        'dim3': slice('a', 'c')}
-        self.assertEqual(data.indexed(**int_slicers),
-                         data.labeled(**loc_slicers))
+        self.assertEqual(data.isel(**int_slicers),
+                         data.sel(**loc_slicers))
         data['time'] = ('time', pd.date_range('2000-01-01', periods=20))
-        self.assertEqual(data.indexed(time=0),
-                         data.labeled(time='2000-01-01'))
-        self.assertEqual(data.indexed(time=slice(10)),
-                         data.labeled(time=slice('2000-01-01',
+        self.assertEqual(data.isel(time=0),
+                         data.sel(time='2000-01-01'))
+        self.assertEqual(data.isel(time=slice(10)),
+                         data.sel(time=slice('2000-01-01',
                                                    '2000-01-10')))
-        self.assertEqual(data, data.labeled(time=slice('1999', '2005')))
-        self.assertEqual(data.indexed(time=slice(3)),
-                         data.labeled(
+        self.assertEqual(data, data.sel(time=slice('1999', '2005')))
+        self.assertEqual(data.isel(time=slice(3)),
+                         data.sel(
                             time=pd.date_range('2000-01-01', periods=3)))
+        self.assertEqual(data.isel(time=slice(3)),
+                         data.sel(time=(data['time.dayofyear'] <= 3)))
 
     def test_reindex_like(self):
         data = create_test_data()
-        expected = data.indexed(dim1=slice(10), time=slice(13))
+        data['letters'] = ('dim3', 10 * ['a'])
+
+        expected = data.isel(dim1=slice(10), time=slice(13))
         actual = data.reindex_like(expected)
         self.assertDatasetIdentical(actual, expected)
 
@@ -253,6 +304,8 @@ class TestDataset(TestCase):
         expected['dim3'] = ('dim3', list('cdefghijkl'))
         expected['var3'][:-2] = expected['var3'][2:]
         expected['var3'][-2:] = np.nan
+        expected['letters'] = expected['letters'].astype(object)
+        expected['letters'][-2:] = np.nan
         actual = data.reindex_like(expected)
         self.assertDatasetIdentical(actual, expected)
 
@@ -260,14 +313,14 @@ class TestDataset(TestCase):
         data = create_test_data()
         self.assertDatasetIdentical(data, data.reindex())
 
-        expected = data.indexed(dim1=slice(10))
+        expected = data.isel(dim1=slice(10))
         actual = data.reindex(dim1=data['dim1'][:10])
         self.assertDatasetIdentical(actual, expected)
 
         actual = data.reindex(dim1=data['dim1'][:10].values)
         self.assertDatasetIdentical(actual, expected)
 
-        actual = data.reindex(dim1=data['dim1'][:10].as_index)
+        actual = data.reindex(dim1=data['dim1'][:10].to_index())
         self.assertDatasetIdentical(actual, expected)
 
     def test_align(self):
@@ -287,23 +340,23 @@ class TestDataset(TestCase):
         left2, right2 = align(left, right, join='outer')
         self.assertVariableEqual(left2['dim3'], right2['dim3'])
         self.assertArrayEqual(left2['dim3'], union)
-        self.assertDatasetIdentical(left2.labeled(dim3=intersection),
-                                    right2.labeled(dim3=intersection))
+        self.assertDatasetIdentical(left2.sel(dim3=intersection),
+                                    right2.sel(dim3=intersection))
         self.assertTrue(np.isnan(left2['var3'][-2:]).all())
         self.assertTrue(np.isnan(right2['var3'][:2]).all())
 
         left2, right2 = align(left, right, join='left')
         self.assertVariableEqual(left2['dim3'], right2['dim3'])
         self.assertVariableEqual(left2['dim3'], left['dim3'])
-        self.assertDatasetIdentical(left2.labeled(dim3=intersection),
-                                    right2.labeled(dim3=intersection))
+        self.assertDatasetIdentical(left2.sel(dim3=intersection),
+                                    right2.sel(dim3=intersection))
         self.assertTrue(np.isnan(right2['var3'][:2]).all())
 
         left2, right2 = align(left, right, join='right')
         self.assertVariableEqual(left2['dim3'], right2['dim3'])
         self.assertVariableEqual(left2['dim3'], right['dim3'])
-        self.assertDatasetIdentical(left2.labeled(dim3=intersection),
-                                    right2.labeled(dim3=intersection))
+        self.assertDatasetIdentical(left2.sel(dim3=intersection),
+                                    right2.sel(dim3=intersection))
         self.assertTrue(np.isnan(left2['var3'][-2:]).all())
 
     def test_variable_indexing(self):
@@ -319,28 +372,28 @@ class TestDataset(TestCase):
         self.assertVariableEqual(v[:3, :2], v[range(3), range(2)])
         self.assertVariableEqual(v[:3, :2], v.loc[d1[:3], d2[:2]])
 
-    def test_select(self):
+    def test_select_vars(self):
         data = create_test_data()
-        ret = data.select(_testvar)
+        ret = data.select_vars(_testvar)
         self.assertVariableEqual(data[_testvar], ret[_testvar])
         self.assertTrue(sorted(_vars.keys())[1] not in ret.variables)
-        self.assertRaises(ValueError, data.select, (_testvar, 'not_a_var'))
+        self.assertRaises(ValueError, data.select_vars, (_testvar, 'not_a_var'))
 
-    def test_unselect(self):
+    def test_drop_vars(self):
         data = create_test_data()
 
-        self.assertEqual(data, data.unselect())
+        self.assertEqual(data, data.drop_vars())
 
-        expected = Dataset({k: data[k] for k in data if k != 'time'})
-        actual = data.unselect('time')
+        expected = Dataset(dict((k, data[k]) for k in data if k != 'time'))
+        actual = data.drop_vars('time')
         self.assertEqual(expected, actual)
 
-        expected = Dataset({k: data[k] for k in ['dim2', 'dim3', 'time']})
-        actual = data.unselect('dim1')
+        expected = Dataset(dict((k, data[k]) for k in ['dim2', 'dim3', 'time']))
+        actual = data.drop_vars('dim1')
         self.assertEqual(expected, actual)
 
-        with self.assertRaisesRegexp(ValueError, 'does not exist in this'):
-            data.unselect('not_found_here')
+        with self.assertRaisesRegexp(ValueError, 'cannot be found'):
+            data.drop_vars('not_found_here')
 
     def test_copy(self):
         data = create_test_data()
@@ -371,7 +424,7 @@ class TestDataset(TestCase):
             variables[v] = variables.pop(k)
 
         for k, v in iteritems(variables):
-            dims = list(v.dimensions)
+            dims = list(v.dims)
             for name, newname in iteritems(newnames):
                 if name in dims:
                     dims[dims.index(name)] = newname
@@ -389,11 +442,23 @@ class TestDataset(TestCase):
 
         # verify that we can rename a variable without accessing the data
         var1 = data['var1']
-        data['var1'] = (var1.dimensions, InaccessibleArray(var1.values))
+        data['var1'] = (var1.dims, InaccessibleArray(var1.values))
         renamed = data.rename(newnames)
         with self.assertRaises(UnexpectedDataAccess):
             renamed['renamed_var1'].values
 
+    def test_rename_inplace(self):
+        times = pd.date_range('2000-01-01', periods=3)
+        data = Dataset({'z': ('x', [2, 3, 4]), 't': ('t', times)})
+        copied = data.copy()
+        renamed = data.rename({'x': 'y'})
+        data.rename({'x': 'y'}, inplace=True)
+        self.assertDatasetIdentical(data, renamed)
+        self.assertFalse(data.equals(copied))
+        self.assertEquals(data.dims, {'y': 3, 't': 3})
+        # check virtual variables
+        self.assertArrayEqual(data['t.dayofyear'], [1, 2, 3])
+
     def test_update(self):
         data = create_test_data(seed=0)
         var2 = Variable('dim1', np.arange(100))
@@ -410,13 +475,13 @@ class TestDataset(TestCase):
 
     def test_merge(self):
         data = create_test_data()
-        ds1 = data.select('var1')
-        ds2 = data.select('var3')
-        expected = data.select('var1', 'var3')
+        ds1 = data.select_vars('var1')
+        ds2 = data.select_vars('var3')
+        expected = data.select_vars('var1', 'var3')
         actual = ds1.merge(ds2)
         self.assertEqual(expected, actual)
         with self.assertRaises(ValueError):
-            ds1.merge(ds2.indexed(dim1=slice(2)))
+            ds1.merge(ds2.isel(dim1=slice(2)))
         with self.assertRaises(ValueError):
             ds1.merge(ds2.rename({'var3': 'var1'}))
 
@@ -424,7 +489,10 @@ class TestDataset(TestCase):
         data = create_test_data()
         self.assertIsInstance(data['var1'], DataArray)
         self.assertVariableEqual(data['var1'], data.variables['var1'])
-        self.assertIs(data['var1'].dataset, data)
+        self.assertDatasetIdentical(data['var1'].dataset, data)
+
+        with self.assertRaises(KeyError):
+            data['notfound']
 
     def test_virtual_variables(self):
         # access virtual variables
@@ -432,7 +500,7 @@ class TestDataset(TestCase):
         self.assertVariableEqual(data['time.dayofyear'],
                                  Variable('time', 1 + np.arange(20)))
         self.assertArrayEqual(data['time.month'].values,
-                              data.variables['time'].as_index.month)
+                              data.variables['time'].to_index().month)
         self.assertArrayEqual(data['time.season'].values, 1)
         # test virtual variable math
         self.assertArrayEqual(data['time.dayofyear'] + 1, 2 + np.arange(20))
@@ -440,7 +508,7 @@ class TestDataset(TestCase):
                               np.sin(1 + np.arange(20)))
         # test slicing the virtual variable -- it should still be virtual
         actual = data['time.dayofyear'][:10].dataset
-        expected = data.indexed(time=slice(10))
+        expected = data.isel(time=slice(10))
         self.assertDatasetIdentical(expected, actual)
 
     def test_slice_virtual_variable(self):
@@ -471,23 +539,27 @@ class TestDataset(TestCase):
         # can't resize a used dimension
         with self.assertRaisesRegexp(ValueError, 'but already exists with'):
             data1['dim1'] = data1['dim1'][:5]
+        # can't use the same dimension name as a scalar var
+        data1['scalar'] = ([], 0)
+        with self.assertRaisesRegexp(ValueError, 'already exists as a scalar'):
+            data1['newvar'] = ('scalar', [3, 4, 5])
 
     def test_delitem(self):
         data = create_test_data()
-        all_items = {'time', 'dim1', 'dim2', 'dim3', 'var1', 'var2', 'var3'}
+        all_items = set(['time', 'dim1', 'dim2', 'dim3', 'var1', 'var2', 'var3'])
         self.assertItemsEqual(data, all_items)
         del data['var1']
-        self.assertItemsEqual(data, all_items - {'var1'})
+        self.assertItemsEqual(data, all_items - set(['var1']))
         del data['dim1']
-        self.assertItemsEqual(data, {'time', 'dim2', 'dim3'})
+        self.assertItemsEqual(data, set(['time', 'dim2', 'dim3']))
 
     def test_squeeze(self):
         data = Dataset({'foo': (['x', 'y', 'z'], [[[1], [2]]])})
         for args in [[], [['x']], [['x', 'z']]]:
             def get_args(v):
-                return [set(args[0]) & set(v.dimensions)] if args else []
-            expected = Dataset({k: v.squeeze(*get_args(v))
-                               for k, v in iteritems(data.variables)})
+                return [set(args[0]) & set(v.dims)] if args else []
+            expected = Dataset(dict((k, v.squeeze(*get_args(v)))
+                                    for k, v in iteritems(data.variables)))
             self.assertDatasetIdentical(expected, data.squeeze(*args))
         # invalid squeeze
         with self.assertRaisesRegexp(ValueError, 'cannot select a dimension'):
@@ -501,9 +573,9 @@ class TestDataset(TestCase):
         self.assertEqual(len(groupby), 3)
         expected_groups = {'a': 0, 'b': 1, 'c': 2}
         self.assertEqual(groupby.groups, expected_groups)
-        expected_items = [('a', data.indexed(x=0)),
-                          ('b', data.indexed(x=1)),
-                          ('c', data.indexed(x=2))]
+        expected_items = [('a', data.isel(x=0)),
+                          ('b', data.isel(x=1)),
+                          ('c', data.isel(x=2))]
         self.assertEqual(list(groupby), expected_items)
 
         identity = lambda x: x
@@ -511,6 +583,7 @@ class TestDataset(TestCase):
             actual = data.groupby(k, squeeze=False).apply(identity)
             self.assertEqual(data, actual)
 
+    def test_groupby_iter(self):
         data = create_test_data()
         for n, (t, sub) in enumerate(list(data.groupby('dim1'))[:3]):
             self.assertEqual(data['dim1'][n], t)
@@ -518,25 +591,49 @@ class TestDataset(TestCase):
             self.assertVariableEqual(data['var2'][n], sub['var2'])
             self.assertVariableEqual(data['var3'][:, n], sub['var3'])
 
-        # TODO: test the other edge cases
+    def test_groupby_errors(self):
+        data = create_test_data()
         with self.assertRaisesRegexp(ValueError, 'must be 1 dimensional'):
             data.groupby('var1')
+        with self.assertRaisesRegexp(ValueError, 'must have a name'):
+            data.groupby(np.arange(10))
         with self.assertRaisesRegexp(ValueError, 'length does not match'):
             data.groupby(data['dim1'][:3])
+        with self.assertRaisesRegexp(ValueError, "must have a 'dims'"):
+            data.groupby(data.coords['dim1'].to_index())
+
+    def test_groupby_reduce(self):
+        data = Dataset({'xy': (['x', 'y'], np.random.randn(3, 4)),
+                        'xonly': ('x', np.random.randn(3)),
+                        'yonly': ('y', np.random.randn(4)),
+                        'letters': ('y', ['a', 'a', 'b', 'b'])})
+
+        expected = data.mean('y')
+        actual = data.groupby('x').mean()
+        self.assertDatasetAllClose(expected, actual)
+
+        actual = data.groupby('x').mean('y')
+        self.assertDatasetAllClose(expected, actual)
+
+        expected = Dataset({'xy': data['xy'].groupby('letters').mean(),
+                            'xonly': data['xonly'].mean(),
+                            'yonly': data['yonly'].groupby('letters').mean()})
+        actual = data.groupby('letters').mean()
+        self.assertDatasetAllClose(expected, actual)
 
     def test_concat(self):
         data = create_test_data()
 
-        split_data = [data.indexed(dim1=slice(10)),
-                      data.indexed(dim1=slice(10, None))]
+        split_data = [data.isel(dim1=slice(10)),
+                      data.isel(dim1=slice(10, None))]
         self.assertDatasetIdentical(data, Dataset.concat(split_data, 'dim1'))
 
         def rectify_dim_order(dataset):
             # return a new dataset with all variable dimensions tranposed into
             # the order in which they are found in `data`
-            return Dataset({k: v.transpose(*data[k].dimensions)
-                           for k, v in iteritems(dataset.variables)},
-                           dataset.attrs)
+            return Dataset(dict((k, v.transpose(*data[k].dims))
+                                for k, v in iteritems(dataset.variables)),
+                           attrs=dataset.attrs)
 
         for dim in ['dim1', 'dim2', 'dim3']:
             datasets = [g for _, g in data.groupby(dim, squeeze=False)]
@@ -548,7 +645,7 @@ class TestDataset(TestCase):
 
             datasets = [g for _, g in data.groupby(dim, squeeze=True)]
             concat_over = [k for k, v in iteritems(data.variables)
-                           if dim in v.dimensions and k != dim]
+                           if dim in v.dims and k != dim]
             actual = Dataset.concat(datasets, data[dim],
                                     concat_over=concat_over)
             self.assertDatasetIdentical(data, rectify_dim_order(actual))
@@ -558,12 +655,12 @@ class TestDataset(TestCase):
 
         # Now add a new variable that doesn't depend on any of the current
         # dims and make sure the mode argument behaves as expected
-        data['var4'] = ('dim4', np.arange(data.dimensions['dim3']))
+        data['var4'] = ('dim4', np.arange(data.dims['dim3']))
         for dim in ['dim1', 'dim2', 'dim3']:
             datasets = [g for _, g in data.groupby(dim, squeeze=False)]
             actual = Dataset.concat(datasets, data[dim], mode='all')
             expected = np.array([data['var4'].values
-                                 for _ in range(data.dimensions[dim])])
+                                 for _ in range(data.dims[dim])])
             self.assertArrayEqual(actual['var4'].values, expected)
 
             actual = Dataset.concat(datasets, data[dim], mode='different')
@@ -571,19 +668,18 @@ class TestDataset(TestCase):
             actual = Dataset.concat(datasets, data[dim], mode='minimal')
             self.assertDataArrayEqual(data['var4'], actual['var4'])
 
-        # verify that the dimension argument takes precedence over
+        # verify that the dim argument takes precedence over
         # concatenating dataset variables of the same name
-        dimension = (2 * data['dim1']).rename('dim1')
+        dim = (2 * data['dim1']).rename('dim1')
         datasets = [g for _, g in data.groupby('dim1', squeeze=False)]
         expected = data.copy()
-        expected['dim1'] = dimension
-        self.assertDatasetIdentical(
-            expected, Dataset.concat(datasets, dimension))
+        expected['dim1'] = dim
+        self.assertDatasetIdentical(expected, Dataset.concat(datasets, dim))
 
         # TODO: factor this into several distinct tests
         data = create_test_data()
-        split_data = [data.indexed(dim1=slice(10)),
-                      data.indexed(dim1=slice(10, None))]
+        split_data = [data.isel(dim1=slice(10)),
+                      data.isel(dim1=slice(10, None))]
 
         with self.assertRaisesRegexp(ValueError, 'must supply at least one'):
             Dataset.concat([], 'dim1')
@@ -643,6 +739,8 @@ class TestDataset(TestCase):
         data = create_test_data()
         roundtripped = pickle.loads(pickle.dumps(data))
         self.assertDatasetIdentical(data, roundtripped)
+        # regression test for #167:
+        self.assertEqual(data.dims, roundtripped.dims)
 
     def test_lazy_load(self):
         store = InaccessibleVariableDataStore()
@@ -650,9 +748,117 @@ class TestDataset(TestCase):
 
         for decode_cf in [False, True]:
             ds = Dataset.load_store(store, decode_cf=decode_cf)
+            with self.assertRaises(UnexpectedDataAccess):
+                ds.load_data()
             with self.assertRaises(UnexpectedDataAccess):
                 ds['var1'].values
 
             # these should not raise UnexpectedDataAccess:
-            ds.indexed(time=10)
-            ds.indexed(time=slice(10), dim1=[0]).indexed(dim1=0, dim2=-1)
+            ds.isel(time=10)
+            ds.isel(time=slice(10), dim1=[0]).isel(dim1=0, dim2=-1)
+
+    def test_reduce(self):
+        data = create_test_data()
+
+        self.assertEqual(len(data.mean().coords), 0)
+
+        expected = data.max()
+        for var in data.noncoords:
+            expected = data[var].max()
+            actual = expected[var]
+            self.assertDataArrayEqual(expected, actual)
+
+        self.assertDatasetEqual(data.min(dim=['dim1']),
+                                data.min(dim='dim1'))
+
+        for reduct, expected in [('dim2', ['dim1', 'dim3', 'time']),
+                                 (['dim2', 'time'], ['dim1', 'dim3']),
+                                 (('dim2', 'time'), ['dim1', 'dim3']),
+                                 ((), ['dim1', 'dim2', 'dim3', 'time'])]:
+            actual = data.min(dim=reduct).dims
+            print(reduct, actual, expected)
+            self.assertItemsEqual(actual, expected)
+
+        self.assertDatasetEqual(data.mean(dim=[]), data)
+
+    def test_reduce_bad_dim(self):
+        data = create_test_data()
+        with self.assertRaisesRegexp(ValueError, 'Dataset does not contain'):
+            ds = data.mean(dim='bad_dim')
+
+    def test_reduce_non_numeric(self):
+        data1 = create_test_data(seed=44)
+        data2 = create_test_data(seed=44)
+        add_vars = {'var4': ['dim1', 'dim2']}
+        for v, dims in sorted(add_vars.items()):
+            size = tuple(_dims[d] for d in dims)
+            data = np.random.random_integers(0, 100, size=size).astype(np.str_)
+            data1[v] = (dims, data, {'foo': 'variable'})
+
+        self.assertTrue('var4' not in data1.mean())
+        self.assertDatasetEqual(data1.mean(), data2.mean())
+        self.assertDatasetEqual(data1.mean(dim='dim1'),
+                                data2.mean(dim='dim1'))
+
+    def test_reduce_keep_attrs(self):
+        data = create_test_data()
+        _attrs = {'attr1': 'value1', 'attr2': 2929}
+
+        attrs = OrderedDict(_attrs)
+        data.attrs = attrs
+
+        # Test dropped attrs
+        ds = data.mean()
+        self.assertEqual(len(ds.attrs), 0)
+        self.assertEqual(ds.attrs, OrderedDict())
+
+        # Test kept attrs
+        ds = data.mean(keep_attrs=True)
+        self.assertEqual(len(ds.attrs), len(_attrs))
+        self.assertTrue(ds.attrs, attrs)
+
+    def test_reduce_argmin(self):
+        # regression test for #205
+        ds = Dataset({'a': ('x', [0, 1])})
+        expected = Dataset({'a': ([], 0)})
+        actual = ds.argmin()
+        self.assertDatasetIdentical(expected, actual)
+
+        actual = ds.argmin('x')
+        self.assertDatasetIdentical(expected, actual)
+
+    @unittest.skip('see github issue 209')
+    def test_reduce_only_one_axis(self):
+
+        def mean_only_one_axis(x, axis):
+            if not isinstance(axis, (int, np.integer)):
+                raise TypeError('non-integer axis')
+            return x.mean(axis)
+
+        ds = Dataset({'a': (['x', 'y'], [[0, 1, 2, 3, 4]])})
+        expected = Dataset({'a': ('x', [2])})
+        actual = ds.reduce(mean_only_one_axis, 'y')
+        self.assertDatasetIdentical(expected, actual)
+
+        with self.assertRaisesRegexp(TypeError, 'non-integer axis'):
+            ds.reduce(mean_only_one_axis)
+
+        with self.assertRaisesRegexp(TypeError, 'non-integer axis'):
+            ds.reduce(mean_only_one_axis, ['x', 'y'])
+
+    def test_apply(self):
+        data = create_test_data()
+        data.attrs['foo'] = 'bar'
+
+        self.assertDatasetIdentical(data.apply(np.mean), data.mean())
+        self.assertDatasetIdentical(data.apply(np.mean, keep_attrs=True),
+                                    data.mean(keep_attrs=True))
+
+        self.assertDatasetIdentical(data.apply(lambda x: x, keep_attrs=True),
+                                    data.drop_vars('time'))
+
+        def scale(x, multiple=1):
+            return multiple * x
+
+        actual = data.apply(scale, multiple=2)
+        self.assertDataArrayEqual(actual['var1'], 2 * data['var1'])
