@@ -1,4 +1,4 @@
-from collections import namedtuple, OrderedDict
+from collections import namedtuple
 from copy import copy, deepcopy
 from datetime import datetime
 from textwrap import dedent
@@ -9,7 +9,7 @@ import pandas as pd
 from xray import Variable, Dataset, DataArray, indexing
 from xray.variable import (Coordinate, as_variable, NumpyArrayAdapter,
                            PandasIndexAdapter, _as_compatible_data)
-from xray.pycompat import PY3
+from xray.pycompat import PY3, OrderedDict
 
 from . import TestCase, source_ndarray
 
@@ -18,7 +18,7 @@ class VariableSubclassTestCases(object):
     def test_properties(self):
         data = 0.5 * np.arange(10)
         v = self.cls(['time'], data, {'foo': 'bar'})
-        self.assertEqual(v.dimensions, ('time',))
+        self.assertEqual(v.dims, ('time',))
         self.assertArrayEqual(v.values, data)
         self.assertEqual(v.dtype, float)
         self.assertEqual(v.shape, (10,))
@@ -164,7 +164,7 @@ class VariableSubclassTestCases(object):
         self.assertArrayEqual((v * w).values, x * y)
         # something complicated
         self.assertArrayEqual((v ** 2 * w - 1 + x).values, x ** 2 * y - 1 + x)
-        # make sure dtype is preserved (for Coordinates)
+        # make sure dtype is preserved (for Index objects)
         self.assertEqual(float, (+v).dtype)
         self.assertEqual(float, (+v).values.dtype)
         self.assertEqual(float, (0 + v).dtype)
@@ -227,11 +227,11 @@ class VariableSubclassTestCases(object):
         v = self.cls(['a'], x)
         w = self.cls(['a'], y)
         self.assertVariableIdentical(Variable(['b', 'a'], np.array([x, y])),
-                               Variable.concat([v, w], 'b'))
+                                     Variable.concat([v, w], 'b'))
         self.assertVariableIdentical(Variable(['b', 'a'], np.array([x, y])),
-                               Variable.concat((v, w), 'b'))
+                                     Variable.concat((v, w), 'b'))
         self.assertVariableIdentical(Variable(['b', 'a'], np.array([x, y])),
-                               Variable.concat((v, w), 'b', length=2))
+                                     Variable.concat((v, w), 'b', length=2))
         with self.assertRaisesRegexp(ValueError, 'actual length'):
             Variable.concat([v, w], 'b', length=1)
         with self.assertRaisesRegexp(ValueError, 'actual length'):
@@ -246,7 +246,19 @@ class VariableSubclassTestCases(object):
         # test dimension order
         self.assertVariableIdentical(v, Variable.concat([v[:, :5], v[:, 5:]], 'x'))
         self.assertVariableIdentical(v.transpose(),
-                               Variable.concat([v[:, 0], v[:, 1:]], 'x'))
+                                     Variable.concat([v[:, 0], v[:, 1:]], 'x'))
+
+    def test_concat_attrs(self):
+        # different or conflicting attributes should be removed
+        v = self.cls('a', np.arange(5), {'foo': 'bar'})
+        w = self.cls('a', np.ones(5))
+        expected = self.cls('a', np.concatenate([np.arange(5), np.ones(5)]))
+        self.assertVariableIdentical(expected, Variable.concat([v, w], 'a'))
+        w.attrs['foo'] = 2
+        self.assertVariableIdentical(expected, Variable.concat([v, w], 'a'))
+        w.attrs['foo'] = 'bar'
+        expected.attrs['foo'] = 'bar'
+        self.assertVariableIdentical(expected, Variable.concat([v, w], 'a'))
 
     def test_copy(self):
         v = self.cls('x', 0.5 * np.arange(10), {'foo': 'bar'})
@@ -282,11 +294,14 @@ class TestVariable(TestCase, VariableSubclassTestCases):
         v.values = d2
         self.assertIs(source_ndarray(v.values), d2)
 
-    def test_item(self):
+    def test_numpy_same_methods(self):
         v = Variable([], np.float32(0.0))
         self.assertEqual(v.item(), 0)
         self.assertIs(type(v.item()), float)
 
+        v = Coordinate('x', np.arange(5))
+        self.assertEqual(2, v.searchsorted(2))
+
     def test_datetime64_conversion(self):
         # verify that datetime64 is always converted to ns precision with
         # sources preserved
@@ -324,9 +339,9 @@ class TestVariable(TestCase, VariableSubclassTestCases):
         d = np.random.rand(10, 3)
         d[0, 0] = np.nan
         v1 = Variable(('dim1', 'dim2'), data=d,
-                       attributes={'att1': 3, 'att2': [1, 2, 3]})
+                       attrs={'att1': 3, 'att2': [1, 2, 3]})
         v2 = Variable(('dim1', 'dim2'), data=d,
-                       attributes={'att1': 3, 'att2': [1, 2, 3]})
+                       attrs={'att1': 3, 'att2': [1, 2, 3]})
         self.assertTrue(v1.equals(v2))
         self.assertTrue(v1.identical(v2))
 
@@ -359,18 +374,19 @@ class TestVariable(TestCase, VariableSubclassTestCases):
         self.assertIsInstance(as_variable(ds['x']), Variable)
         self.assertIsInstance(as_variable(ds['x'], strict=False), DataArray)
 
-        FakeVariable = namedtuple('FakeVariable', 'values dimensions')
-        fake_xarray = FakeVariable(expected.values, expected.dimensions)
+        FakeVariable = namedtuple('FakeVariable', 'values dims')
+        fake_xarray = FakeVariable(expected.values, expected.dims)
         self.assertVariableIdentical(expected, as_variable(fake_xarray))
 
-        xarray_tuple = (expected.dimensions, expected.values)
+        xarray_tuple = (expected.dims, expected.values)
         self.assertVariableIdentical(expected, as_variable(xarray_tuple))
 
         with self.assertRaisesRegexp(TypeError, 'cannot convert numpy'):
             as_variable(data)
-        with self.assertRaisesRegexp(TypeError, 'cannot convert arg'):
+        with self.assertRaisesRegexp(TypeError, 'can only convert tuples'):
             as_variable(list(data))
-
+        with self.assertRaisesRegexp(TypeError, 'cannot convert arg'):
+            as_variable(tuple(data))
 
     def test_repr(self):
         v = Variable(['time', 'x'], [[1, 2, 3], [4, 5, 6]], {'foo': 'bar'})
@@ -415,14 +431,19 @@ class TestVariable(TestCase, VariableSubclassTestCases):
         v[range(10), range(11)] = 1
         self.assertArrayEqual(v.values, np.ones((10, 11)))
 
-    def test_indexed(self):
+    def test_isel(self):
         v = Variable(['time', 'x'], self.d)
-        self.assertVariableIdentical(v.indexed(time=slice(None)), v)
-        self.assertVariableIdentical(v.indexed(time=0), v[0])
-        self.assertVariableIdentical(v.indexed(time=slice(0, 3)), v[:3])
-        self.assertVariableIdentical(v.indexed(x=0), v[:, 0])
+        self.assertVariableIdentical(v.isel(time=slice(None)), v)
+        self.assertVariableIdentical(v.isel(time=0), v[0])
+        self.assertVariableIdentical(v.isel(time=slice(0, 3)), v[:3])
+        self.assertVariableIdentical(v.isel(x=0), v[:, 0])
         with self.assertRaisesRegexp(ValueError, 'do not exist'):
-            v.indexed(not_a_dim=0)
+            v.isel(not_a_dim=0)
+
+    def test_index_0d_numpy_string(self):
+        # regression test to verify our work around for indexing 0d strings
+        v = Variable([], np.string_('asdf'))
+        self.assertVariableIdentical(v[()], v)
 
     def test_transpose(self):
         v = Variable(['time', 'x'], self.d)
@@ -517,7 +538,7 @@ class TestVariable(TestCase, VariableSubclassTestCases):
         self.assertVariableIdentical(v.reduce(np.std, 'x'),
                                      Variable(['y'], self.d.std(axis=0)))
         self.assertVariableIdentical(v.reduce(np.std, axis=0),
-                                     v.reduce(np.std, dimension='x'))
+                                     v.reduce(np.std, dim='x'))
         self.assertVariableIdentical(v.reduce(np.std, ['y', 'x']),
                                      Variable([], self.d.std(axis=(0, 1))))
         self.assertVariableIdentical(v.reduce(np.std),
@@ -528,7 +549,22 @@ class TestVariable(TestCase, VariableSubclassTestCases):
         self.assertVariableIdentical(v.mean('x'), v.reduce(np.mean, 'x'))
 
         with self.assertRaisesRegexp(ValueError, 'cannot supply both'):
-            v.mean(dimension='x', axis=0)
+            v.mean(dim='x', axis=0)
+
+    def test_reduce_keep_attrs(self):
+        _attrs = {'units': 'test', 'long_name': 'testing'}
+
+        v = Variable(['x', 'y'], self.d, _attrs)
+
+        # Test dropped attrs
+        vm = v.mean()
+        self.assertEqual(len(vm.attrs), 0)
+        self.assertEqual(vm.attrs, OrderedDict())
+
+        # Test kept attrs
+        vm = v.mean(keep_attrs=True)
+        self.assertEqual(len(vm.attrs), len(_attrs))
+        self.assertEqual(vm.attrs, _attrs)
 
 
 class TestCoordinate(TestCase, VariableSubclassTestCases):
@@ -538,10 +574,10 @@ class TestCoordinate(TestCase, VariableSubclassTestCases):
         with self.assertRaisesRegexp(ValueError, 'must be 1-dimensional'):
             Coordinate((), 0)
 
-    def test_as_index(self):
+    def test_to_index(self):
         data = 0.5 * np.arange(10)
         v = Coordinate(['time'], data, {'foo': 'bar'})
-        self.assertTrue(pd.Index(data).equals(v.as_index))
+        self.assertTrue(pd.Index(data, name='time').identical(v.to_index()))
 
     def test_data(self):
         x = Coordinate('x', np.arange(3.0))
@@ -550,7 +586,7 @@ class TestCoordinate(TestCase, VariableSubclassTestCases):
         self.assertEqual(float, x.dtype)
         self.assertArrayEqual(np.arange(3), x)
         self.assertEqual(float, x.values.dtype)
-        # after inspecting x.values, the Coordinate will be saved as an Index
+        # after inspecting x.values, the Coordinate value will be saved as an Index
         self.assertIsInstance(x._data, PandasIndexAdapter)
         with self.assertRaisesRegexp(TypeError, 'cannot be modified'):
             x[:] = 0
@@ -567,6 +603,13 @@ class TestCoordinate(TestCase, VariableSubclassTestCases):
         self.assertEqual(t.dtype, object)
         self.assertEqual(t[:2].dtype, object)
 
+    def test_name(self):
+        coord = Coordinate('x', [10.0])
+        self.assertEqual(coord.name, 'x')
+
+        with self.assertRaises(AttributeError):
+            coord.name = 'y'
+
 
 class TestAsCompatibleData(TestCase):
     def test_unchanged_types(self):
