@@ -1,7 +1,24 @@
+import numpy as np
+
 from xray.utils import FrozenOrderedDict
 from xray.pycompat import iteritems
 
 
+NONE_VAR_NAME = '__values__'
+
+
+def _encode_variable_name(name):
+    if name is None:
+        name = NONE_VAR_NAME
+    return name
+
+
+def _decode_variable_name(name):
+    if name == NONE_VAR_NAME:
+        name = None
+    return name
+
+
 class AbstractDataStore(object):
     def open_store_variable(self, v):
         raise NotImplementedError
@@ -12,7 +29,8 @@ class AbstractDataStore(object):
 
     @property
     def variables(self):
-        return FrozenOrderedDict((k, self.open_store_variable(v))
+        return FrozenOrderedDict((_decode_variable_name(k),
+                                  self.open_store_variable(v))
                                  for k, v in iteritems(self.store_variables))
 
     def sync(self):
@@ -39,9 +57,9 @@ class AbstractWritableDataStore(AbstractDataStore):
 
     def set_variables(self, variables):
         for vn, v in iteritems(variables):
-            self.set_variable(vn, v)
+            self.set_variable(_encode_variable_name(vn), v)
 
     def set_necessary_dimensions(self, variable):
-        for d, l in zip(variable.dimensions, variable.shape):
+        for d, l in zip(variable.dims, variable.shape):
             if d not in self.ds.dimensions:
                 self.set_dimension(d, l)
