@@ -1,22 +1,42 @@
+from collections import Mapping
+
 import numpy as np
 
 from .pycompat import basestring, iteritems
+from . import utils
 
-class ImplementsReduce(object):
+
+class ImplementsArrayReduce(object):
     @classmethod
-    def _reduce_method(cls, f, name=None, module=None):
-        def func(self, dimension=None, axis=None, **kwargs):
-            return self.reduce(f, dimension, axis, **kwargs)
-        if name is None:
-            name = f.__name__
-        func.__name__ = name
-        func.__doc__ = cls._reduce_method_docstring.format(
-            name=('' if module is None else module + '.') + name,
-            cls=cls.__name__)
-        return func
+    def _reduce_method(cls, func):
+        def wrapped_func(self, dim=None, axis=None, keep_attrs=False,
+                         **kwargs):
+            return self.reduce(func, dim, axis, keep_attrs, **kwargs)
+        return wrapped_func
+
+    _reduce_extra_args_docstring = \
+        """dim : str or sequence of str, optional
+            Dimension(s) over which to apply `{name}`.
+        axis : int or sequence of int, optional
+            Axis(es) over which to apply `{name}`. Only one of the 'dim'
+            and 'axis' arguments can be supplied. If neither are supplied, then
+            `{name}` is calculated over axes.\n"""
 
 
-class AbstractArray(ImplementsReduce):
+class ImplementsDatasetReduce(object):
+    @classmethod
+    def _reduce_method(cls, func):
+        def wrapped_func(self, dim=None, keep_attrs=False, **kwargs):
+            return self.reduce(func, dim, keep_attrs, **kwargs)
+        return wrapped_func
+
+    _reduce_extra_args_docstring = \
+        """dim : str or sequence of str, optional
+            Dimension(s) over which to apply `func`.  By default `func` is
+            applied over all dimensions.\n"""
+
+
+class AbstractArray(ImplementsArrayReduce):
     def __nonzero__(self):
         return bool(self.values)
 
@@ -38,10 +58,6 @@ class AbstractArray(ImplementsReduce):
     def __array__(self, dtype=None):
         return np.asarray(self.values, dtype=dtype)
 
-    def item(self):
-        """Calls numpy.ndarray.item on this array's values"""
-        return self.values.item()
-
     def __repr__(self):
         return array_repr(self)
 
@@ -58,12 +74,12 @@ class AbstractArray(ImplementsReduce):
     def T(self):
         return self.transpose()
 
-    def get_axis_num(self, dimension):
+    def get_axis_num(self, dim):
         """Return axis number(s) corresponding to dimension(s) in this array.
 
         Parameters
         ----------
-        dimension : str or iterable of str
+        dim : str or iterable of str
             Dimension name(s) for which to lookup axes.
 
         Returns
@@ -71,43 +87,52 @@ class AbstractArray(ImplementsReduce):
         int or tuple of int
             Axis number or numbers corresponding to the given dimensions.
         """
-        if isinstance(dimension, basestring):
-            return self._get_axis_num(dimension)
+        if isinstance(dim, basestring):
+            return self._get_axis_num(dim)
         else:
-            return tuple(self._get_axis_num(dim) for dim in dimension)
+            return tuple(self._get_axis_num(d) for d in dim)
 
     def _get_axis_num(self, dim):
         try:
-            return self.dimensions.index(dim)
+            return self.dims.index(dim)
         except ValueError:
             raise ValueError("%r not found in array dimensions %r" %
-                             (dim, self.dimensions))
+                             (dim, self.dims))
 
-    _reduce_method_docstring = \
-        """Reduce this {cls}'s data' by applying `{name}` along some
-        dimension(s).
 
-        Parameters
-        ----------
-        dimension : str or sequence of str, optional
-            Dimension(s) over which to apply `{name}`.
-        axis : int or sequence of int, optional
-            Axis(es) over which to apply `{name}`. Only one of the 'dimension'
-            and 'axis' arguments can be supplied. If neither are supplied, then
-            `{name}` is calculated over the flattened array (by calling
-            `{name}(x)` without an axis argument).
-        **kwargs : dict
-            Additional keyword arguments passed on to `{name}`.
+class AbstractCoordinates(Mapping):
+    def __init__(self, data):
+        self._data = data
 
-        Returns
-        -------
-        reduced : {cls}
-            New {cls} object with `{name}` applied to its data and the
-            indicated dimension(s) removed.
-        """
+    def __getitem__(self, key):
+        raise NotImplementedError
 
-    _reduce_dimension_default = None
-    _reduce_axis_default = None
+    def __iter__(self):
+        return iter(self._data.dims)
+
+    def __len__(self):
+        return len(self._data.dims)
+
+    def __contains__(self, key):
+        return key in self._data.dims
+
+    def __repr__(self):
+        return '\n'.join(_wrap_indent(repr(v.to_index()), '%s: ' % k)
+                         for k, v in self.items())
+
+    @staticmethod
+    def _convert_to_coord(key, value, expected_size=None):
+        from .variable import Coordinate, as_variable
+
+        if not isinstance(value, AbstractArray):
+            value = Coordinate(key, value)
+        coord = as_variable(value).to_coord()
+
+        if expected_size is not None and coord.size != expected_size:
+            raise ValueError('new coordinate has size %s but the existing '
+                             'coordinate has size %s'
+                             % (coord.size, expected_size))
+        return coord
 
 
 def _summarize_attributes(data):
@@ -119,15 +144,34 @@ def _summarize_attributes(data):
     return attr_summary
 
 
+def _wrap_indent(text, start='', length=None):
+    if length is None:
+        length = len(start)
+    indent = '\n' + ' ' * length
+    return start + indent.join(x for x in text.splitlines())
+
+
 def array_repr(arr):
-    name_str = ('%r ' % arr.name) if hasattr(arr, 'name') else ''
+    if hasattr(arr, 'name') and arr.name is not None:
+        name_str = '%r ' % arr.name
+    else:
+        name_str = ''
     dim_summary = ', '.join('%s: %s' % (k, v) for k, v
-                            in zip(arr.dimensions, arr.shape))
+                            in zip(arr.dims, arr.shape))
     summary = ['<xray.%s %s(%s)>'% (type(arr).__name__, name_str, dim_summary)]
     if arr.size < 1e5 or arr._in_memory():
         summary.append(repr(arr.values))
     else:
         summary.append('[%s values with dtype=%s]' % (arr.size, arr.dtype))
+    if hasattr(arr, 'dataset'):
+        if arr.coords:
+            summary.append('Coordinates:')
+            summary.append(_wrap_indent(repr(arr.coords), '    '))
+        other_vars = [k for k in arr.dataset
+                      if k not in arr.coords and k != arr.name]
+        if other_vars:
+            summary.append('Linked dataset variables:')
+            summary.append('    ' + ', '.join(other_vars))
     summary.append('Attributes:\n%s' % _summarize_attributes(arr))
     return '\n'.join(summary)
 
@@ -150,18 +194,18 @@ def dataset_repr(ds):
     max_name_length = max(len(k) for k in ds.variables) if ds else 0
     first_col_width = max(4 + max_name_length, 16)
     coords_str = pretty_print('Dimensions:', first_col_width)
-    all_dim_strings = ['%s: %s' % (k, v) for k, v in iteritems(ds.dimensions)]
+    all_dim_strings = ['%s: %s' % (k, v) for k, v in iteritems(ds.dims)]
     summary.append('%s(%s)' % (coords_str, ', '.join(all_dim_strings)))
 
     def summarize_var(k, not_found=' ', found=int):
         v = ds.variables[k]
         dim_strs = []
-        for n, d in enumerate(ds.dimensions):
+        for n, d in enumerate(ds.dims):
             length = len(all_dim_strings[n])
             prepend = ' ' * (length // 2)
-            if d in v.dimensions:
+            if d in v.dims:
                 if found is int:
-                    indicator = str(v.dimensions.index(d))
+                    indicator = str(v.dims.index(d))
                 else:
                     indicator = found
             else:
@@ -178,10 +222,10 @@ def dataset_repr(ds):
             return ['    None']
 
     summary.append('Coordinates:')
-    summary.extend(summarize_variables(ds.coordinates, ' ', 'X'))
+    summary.extend(summarize_variables(ds.coords, ' ', 'X'))
 
     summary.append('Noncoordinates:')
-    summary.extend(summarize_variables(ds.noncoordinates, ' ', int))
+    summary.extend(summarize_variables(ds.noncoords, ' ', int))
 
     summary.append('Attributes:\n%s' % _summarize_attributes(ds))
 
