@@ -1,19 +1,20 @@
 import numpy as np
 import pandas as pd
 import warnings
-from collections import defaultdict, OrderedDict
+from collections import defaultdict
 from datetime import datetime
 
 from . import indexing
 from . import utils
-from .pycompat import iteritems
+from .pycompat import iteritems, bytes_type, unicode_type, OrderedDict
 import xray
 
 # standard calendars recognized by netcdftime
-_STANDARD_CALENDARS = {'standard', 'gregorian', 'proleptic_gregorian'}
+_STANDARD_CALENDARS = set(['standard', 'gregorian', 'proleptic_gregorian'])
 
 
-def mask_and_scale(array, fill_value=None, scale_factor=None, add_offset=None):
+def mask_and_scale(array, fill_value=None, scale_factor=None, add_offset=None,
+                   dtype=float):
     """Scale and mask array values according to CF conventions for packed and
     missing values
 
@@ -44,9 +45,9 @@ def mask_and_scale(array, fill_value=None, scale_factor=None, add_offset=None):
     ----------
     http://www.unidata.ucar.edu/software/netcdf/docs/BestPractices.html
     """
-    # cast to float to insure NaN is meaningful
-    values = np.array(array, dtype=float, copy=True)
-    if fill_value is not None and not np.isnan(fill_value):
+    # by default, cast to float to ensure NaN is meaningful
+    values = np.array(array, dtype=dtype, copy=True)
+    if fill_value is not None and not pd.isnull(fill_value):
         if values.ndim > 0:
             values[values == fill_value] = np.nan
         elif values == fill_value:
@@ -229,7 +230,7 @@ class MaskedAndScaledArray(utils.NDArrayMixin):
     http://www.unidata.ucar.edu/software/netcdf/docs/BestPractices.html
     """
     def __init__(self, array, fill_value=None, scale_factor=None,
-                 add_offset=None):
+                 add_offset=None, dtype=float):
         """
         Parameters
         ----------
@@ -248,20 +249,21 @@ class MaskedAndScaledArray(utils.NDArrayMixin):
         self.fill_value = fill_value
         self.scale_factor = scale_factor
         self.add_offset = add_offset
+        self._dtype = dtype
 
     @property
     def dtype(self):
-        return np.dtype('float')
+        return np.dtype(self._dtype)
 
     def __getitem__(self, key):
         return mask_and_scale(self.array[key], self.fill_value,
-                              self.scale_factor, self.add_offset)
+                              self.scale_factor, self.add_offset, self._dtype)
 
     def __repr__(self):
-        return ("%s(%r, fill_value=%r, scale_factor=%r, add_offset=%r)" %
+        return ("%s(%r, fill_value=%r, scale_factor=%r, add_offset=%r, "
+                "dtype=%r)" %
                 (type(self).__name__, self.array, self.fill_value,
-                 self.scale_factor, self.add_offset))
-
+                 self.scale_factor, self.add_offset, self._dtype))
 
 
 class DecodedCFDatetimeArray(utils.NDArrayMixin):
@@ -279,7 +281,7 @@ class DecodedCFDatetimeArray(utils.NDArrayMixin):
         return np.dtype('datetime64[ns]')
 
     def __getitem__(self, key):
-        return decode_cf_datetime(self.array, units=self.units,
+        return decode_cf_datetime(self.array[key], units=self.units,
                                   calendar=self.calendar)
 
 
@@ -325,7 +327,7 @@ class CharToStringArray(utils.NDArrayMixin):
         else:
             # require slicing the last dimension completely
             key = indexing.expanded_indexer(key, self.array.ndim)
-            if  key[-1] != slice(None):
+            if key[-1] != slice(None):
                 raise IndexError('too many indices')
             values = char_to_string(self.array[key])
         return values
@@ -352,62 +354,115 @@ def char_to_string(arr):
     return arr.view(kind + str(arr.shape[-1]))[..., 0]
 
 
+def _safe_setitem(dest, key, value):
+    if key in dest:
+        raise ValueError('Failed hard to prevent overwriting key %r' % key)
+    dest[key] = value
+
+
+def pop_to(source, dest, key, default=None):
+    """
+    A convenience function which pops a key k from source to dest.
+    None values are not passed on.  If k already exists in dest an
+    error is raised.
+    """
+    value = source.pop(key, default)
+    if value is not None:
+        _safe_setitem(dest, key, value)
+    return value
+
+
+def _infer_dtype(array):
+    """Given an object array with no missing values, infer its dtype from its
+    first element
+    """
+    if array.size == 0:
+        dtype = np.dtype(float)
+    else:
+        dtype = np.array(array.flat[0]).dtype
+        if dtype.kind in ['S', 'U']:
+            # don't just use inferred_dtype to avoid truncating arrays to
+            # the length of their first element
+            dtype = np.dtype(dtype.kind)
+        elif dtype.kind == 'O':
+            raise ValueError('unable to infer dtype; xray cannot '
+                             'serialize arbitrary Python objects')
+    return dtype
+
+
 def encode_cf_variable(var):
     """Converts an Variable into an Variable suitable for saving as a netCDF
     variable
     """
-    dimensions = var.dimensions
+    dimensions = var.dims
     data = var.values
     attributes = var.attrs.copy()
     encoding = var.encoding.copy()
 
+    # convert datetimes into numbers
     if (np.issubdtype(data.dtype, np.datetime64)
             or (data.dtype.kind == 'O'
                 and isinstance(data.reshape(-1)[0], datetime))):
-        # encode datetime arrays into numeric arrays
+        if 'units' in attributes or 'calendar' in attributes:
+            raise ValueError(
+                "Failed hard to prevent overwriting 'units' or 'calendar'")
         (data, units, calendar) = encode_cf_datetime(
             data, encoding.pop('units', None), encoding.pop('calendar', None))
         attributes['units'] = units
         attributes['calendar'] = calendar
-    elif data.dtype.kind == 'O':
-        # Occasionally, one will end up with variables with dtype=object
-        # (likely because they were created from pandas objects which don't
-        # maintain dtype careful). This code makes a best effort attempt to
-        # encode them into a dtype that NETCDF can handle by inspecting the
-        # dtype of the first element.
-        dtype = np.array(data.reshape(-1)[0]).dtype
-        # N.B. the "astype" call below will fail if data cannot be cast to the
-        # type of its first element (which is probably the only sensible thing
-        # to do).
-        data = np.asarray(data).astype(dtype)
-
-    def get_to(source, dest, k):
-        v = source.get(k)
-        dest[k] = v
-        return v
 
     # unscale/mask
     if any(k in encoding for k in ['add_offset', 'scale_factor']):
         data = np.array(data, dtype=float, copy=True)
         if 'add_offset' in encoding:
-            data -= get_to(encoding, attributes, 'add_offset')
+            data -= pop_to(encoding, attributes, 'add_offset')
         if 'scale_factor' in encoding:
-            data /= get_to(encoding, attributes, 'scale_factor')
+            data /= pop_to(encoding, attributes, 'scale_factor')
 
     # replace NaN with the fill value
     if '_FillValue' in encoding:
-        if encoding['_FillValue'] is np.nan:
-            attributes['_FillValue'] = np.nan
-        else:
-            nans = np.isnan(data)
-            if nans.any():
-                data[nans] = get_to(encoding, attributes, '_FillValue')
+        fill_value = pop_to(encoding, attributes, '_FillValue')
+        if not pd.isnull(fill_value):
+            missing = pd.isnull(data)
+            if missing.any():
+                data = data.copy()
+                data[missing] = fill_value
 
-    # restore original dtype
-    if 'dtype' in encoding and encoding['dtype'].kind != 'O':
-        if np.issubdtype(encoding['dtype'], int):
-            data = data.round()
-        data = data.astype(encoding['dtype'])
+    # cast to encoded dtype
+    if 'dtype' in encoding:
+        dtype = np.dtype(encoding.pop('dtype'))
+        if dtype.kind != 'O':
+            if np.issubdtype(dtype, int):
+                data = data.round()
+            if dtype == 'S1' and data.dtype != 'S1':
+                data = string_to_char(np.asarray(data, 'S'))
+                dimensions = dimensions + ('string%s' % data.shape[-1],)
+            data = np.asarray(data, dtype=dtype)
+
+    # infer a valid dtype if necessary
+    # TODO: move this from conventions to backends (it's not CF related)
+    if data.dtype.kind == 'O':
+        missing = pd.isnull(data)
+        if missing.any():
+            non_missing_data = data[~missing]
+            inferred_dtype = _infer_dtype(non_missing_data)
+
+            if inferred_dtype.kind in ['S', 'U']:
+                # There is no safe bit-pattern for NA in typical binary string
+                # formats, we so can't set a _FillValue. Unfortunately, this
+                # means we won't be able to restore string arrays with missing
+                # values.
+                fill_value = ''
+            else:
+                # insist on using float for numeric data
+                if not np.issubdtype(inferred_dtype, float):
+                    inferred_dtype = np.dtype(float)
+                fill_value = np.nan
+
+            data = np.array(data, dtype=inferred_dtype, copy=True)
+            data[missing] = fill_value
+        else:
+            data = np.asarray(data, dtype=_infer_dtype(data))
 
     return xray.Variable(dimensions, data, attributes, encoding=encoding)
 
@@ -415,26 +470,14 @@ def encode_cf_variable(var):
 def decode_cf_variable(var, concat_characters=True, mask_and_scale=True,
                        decode_times=True):
     # use _data instead of data so as not to trigger loading data
+    var = xray.variable.as_variable(var)
     data = var._data
-    dimensions = var.dimensions
+    dimensions = var.dims
     attributes = var.attrs.copy()
     encoding = var.encoding.copy()
 
-    def pop_to(source, dest, k):
-        """
-        A convenience function which pops a key k from source to dest.
-        None values are not passed on.  If k already exists in dest an
-        error is raised.
-        """
-        v = source.pop(k, None)
-        if v is not None:
-            if k in dest:
-                raise ValueError("Failed hard to prevent overwriting key %s" % k)
-            dest[k] = v
-        return v
-
     if 'dtype' in encoding:
-        if var.data.dtype != encoding['dtype']:
+        if data.dtype != encoding['dtype']:
             raise ValueError("Refused to overwrite dtype")
     encoding['dtype'] = data.dtype
 
@@ -447,10 +490,14 @@ def decode_cf_variable(var, concat_characters=True, mask_and_scale=True,
         fill_value = pop_to(attributes, encoding, '_FillValue')
         scale_factor = pop_to(attributes, encoding, 'scale_factor')
         add_offset = pop_to(attributes, encoding, 'add_offset')
-        if ((fill_value is not None and not np.isnan(fill_value))
+        if ((fill_value is not None and not pd.isnull(fill_value))
                 or scale_factor is not None or add_offset is not None):
+            if isinstance(fill_value, (bytes_type, unicode_type)):
+                dtype = object
+            else:
+                dtype = float
             data = MaskedAndScaledArray(data, fill_value, scale_factor,
-                                        add_offset)
+                                        add_offset, dtype)
 
     if decode_times:
         if 'units' in attributes and 'since' in attributes['units']:
@@ -468,7 +515,7 @@ def decode_cf_variables(variables, concat_characters=True, mask_and_scale=True,
     """
     dimensions_used_by = defaultdict(list)
     for v in variables.values():
-        for d in v.dimensions:
+        for d in v.dims:
             dimensions_used_by[d].append(v)
 
     def stackable(dim):
@@ -476,14 +523,14 @@ def decode_cf_variables(variables, concat_characters=True, mask_and_scale=True,
         if dim in variables:
             return False
         for v in dimensions_used_by[dim]:
-            if v.dtype.kind != 'S' or dim != v.dimensions[-1]:
+            if v.dtype.kind != 'S' or dim != v.dims[-1]:
                 return False
         return True
 
     new_vars = OrderedDict()
     for k, v in iteritems(variables):
         concat = (concat_characters and v.dtype.kind == 'S' and v.ndim > 0 and
-                  stackable(v.dimensions[-1]))
+                  stackable(v.dims[-1]))
         new_vars[k] = decode_cf_variable(
             v, concat_characters=concat, mask_and_scale=mask_and_scale,
             decode_times=decode_times)
