@@ -1,8 +1,10 @@
+import contextlib
 import functools
 import operator
 import warnings
-from collections import defaultdict, OrderedDict
+from collections import defaultdict
 
+import numpy as np
 import pandas as pd
 
 import xray
@@ -11,9 +13,53 @@ from . import groupby
 from . import ops
 from . import utils
 from . import variable
-from .common import AbstractArray
-from .utils import FrozenOrderedDict, multi_index_from_product
-from .pycompat import iteritems, basestring
+from .common import AbstractArray, AbstractCoordinates
+from .utils import multi_index_from_product
+from .pycompat import iteritems, basestring, OrderedDict
+
+
+def _is_dict_like(value):
+    return hasattr(value, '__getitem__') and hasattr(value, 'keys')
+
+
+def _infer_coords_and_dims(shape, coords, dims):
+    """All the logic for creating a new DataArray"""
+
+    if isinstance(dims, basestring):
+        dims = [dims]
+
+    if _is_dict_like(coords):
+        if dims is None:
+            dims = list(coords.keys())
+        else:
+            bad_coords = [dim for dim in coords if dim not in dims]
+            if bad_coords:
+                raise ValueError('coordinates %r are not array dimensions'
+                                 % bad_coords)
+        coords = [coords.get(d, None) for d in dims]
+    elif coords is not None and len(coords) != len(shape):
+        raise ValueError('%s coordinates supplied but data has ndim=%s'
+                         % (len(coords), len(shape)))
+
+    if dims is None:
+        dims = ['dim_%s' % n for n in range(len(shape))]
+        if coords is not None:
+            for n, idx in enumerate(coords):
+                if hasattr(idx, 'name') and idx.name is not None:
+                    dims[n] = idx.name
+    else:
+        for d in dims:
+            if not isinstance(d, basestring):
+                raise TypeError('dimension %s is not a string' % d)
+
+    if coords is None:
+        coords = [None] * len(shape)
+    coords = [idx if isinstance(idx, AbstractArray) else
+              variable.Coordinate(dims[n], idx) if idx is not None else
+              variable.Coordinate(dims[n], np.arange(shape[n]))
+              for n, idx in enumerate(coords)]
+
+    return coords, dims
 
 
 class _LocIndexer(object):
@@ -24,7 +70,7 @@ class _LocIndexer(object):
         label_indexers = self.data_array._key_to_indexers(key)
         indexers = []
         for dim, label in iteritems(label_indexers):
-            index = self.data_array.coordinates[dim]
+            index = self.data_array.coords[dim]
             indexers.append(indexing.convert_label_indexer(index, label))
         return tuple(indexers)
 
@@ -35,6 +81,34 @@ class _LocIndexer(object):
         self.data_array[self._remap_key(key)] = value
 
 
+def _assert_coordinates_same_size(orig, new):
+    if not new.size == orig.size:
+        raise ValueError('new coordinate has size %s but the existing '
+                         'coordinate has size %s' % (new.size, orig.size))
+
+
+class DataArrayCoordinates(AbstractCoordinates):
+    """Dictionary like container for DataArray coordinates.
+
+    Essentially an immutable OrderedDict with keys given by the array's
+    dimensions and the values given by the corresponding xray.Coordinate
+    objects.
+    """
+    def __getitem__(self, key):
+        if key in self._data.dims:
+            return self._data.dataset.variables[key]
+        else:
+            raise KeyError(key)
+
+    def __setitem__(self, key, value):
+        if key not in self:
+            raise KeyError('%s is not an existing coordinate')
+
+        coord = self._convert_to_coord(key, value, self[key].size)
+        with self._data._set_new_dataset() as ds:
+            ds._variables[key] = coord
+
+
 class DataArray(AbstractArray):
     """N-dimensional array with labeled coordinates and dimensions.
 
@@ -65,36 +139,81 @@ class DataArray(AbstractArray):
 
     Attributes
     ----------
-    dimensions : tuple
+    dims : tuple
         Dimension names associated with this array.
     values : np.ndarray
         Access or modify DataArray values as a numpy array.
-    coordinates : OrderedDict
-        Dictionary of DataArray objects that label values along each dimension.
+    coords : dict-like
+        Dictionary of Coordinate objects that label values along each dimension.
     """
-    def __new__(cls, dataset, name):
-        warnings.warn('the constructor for DataArray objects will change in a '
-                      'future version of xray. For now, create new '
-                      'DataArray objects by getting items from a Dataset '
-                      'instead of calling the constructor directly.',
-                      FutureWarning, stacklevel=2)
-        return cls._constructor(dataset, name)
-
-    @classmethod
-    def _constructor(cls, dataset, name):
+    def __init__(self, data=None, coords=None, dims=None, name=None,
+                 attrs=None, encoding=None):
         """
         Parameters
         ----------
-        dataset : xray.Dataset
-            The dataset in which to find this array.
-        name : str
-            The name of the variable in `dataset` to which array operations
-            should be applied.
+        data : array_like, optional
+            Values for this array. Must be a ``numpy.ndarray``, ndarray like,
+            or castable to an ``ndarray``. If a self-described xray or pandas
+            object, attempst are made to use this array's metadata to fill in
+            other unspecified arguments. This argument is required unless the
+            'dataset' argument is provided.
+        coords : sequence or dict of array_like objects, optional
+            Coordinates (tick labels) to use for indexing along each dimension.
+            If dict-like, should be a mapping from dimension names to the
+            corresponding coordinates.
+        dims : str or sequence of str, optional
+            Name(s) of the the data dimension(s). Must be either a string (only
+            for 1D data) or a sequence of strings with length equal to the
+            number of dimensions. If this argument is omited, dimension names
+            are taken from ``coords`` (if possible) and otherwise default to
+            ``['dim_0', ... 'dim_n']``.
+        name : str or None, optional
+            Name of this array.
+        attrs : dict_like or None, optional
+            Attributes to assign to the new variable. By default, an empty
+            attribute dictionary is initialized.
+        encoding : dict_like or None, optional
+            Dictionary specifying how to encode this array's data into a
+            serialized format like netCDF4. Currently used keys (for netCDF)
+            include '_FillValue', 'scale_factor', 'add_offset', 'dtype',
+            'units' and 'calendar' (the later two only for datetime arrays).
+            Unrecognized keys are ignored.
+        """
+        # try to fill in arguments from data if they weren't supplied
+        if coords is None:
+            coords = getattr(data, 'coords', None)
+            if isinstance(data, pd.Series):
+                coords = [data.index]
+            elif isinstance(data, pd.DataFrame):
+                coords = [data.index, data.columns]
+            elif isinstance(data, (pd.Index, variable.Coordinate)):
+                coords = [data]
+            elif isinstance(data, pd.Panel):
+                coords = [data.items, data.major_axis, data.minor_axis]
+        if dims is None:
+            dims = getattr(data, 'dims', None)
+        if name is None:
+            name = getattr(data, 'name', None)
+        if attrs is None:
+            attrs = getattr(data, 'attrs', None)
+        if encoding is None:
+            encoding = getattr(data, 'encoding', None)
+
+        data = variable._as_compatible_data(data)
+        coords, dims = _infer_coords_and_dims(data.shape, coords, dims)
+        variables = OrderedDict((var.name, var) for var in coords)
+        variables[name] = variable.Variable(dims, data, attrs, encoding)
+        dataset = xray.Dataset(variables)
+
+        self._dataset = dataset
+        self._name = name
+
+    @classmethod
+    def _new_from_dataset(cls, dataset, name):
+        """Private constructor for the benefit Dataset.__getitem__ (skips all
+        validation)
         """
         obj = object.__new__(cls)
-        if name not in dataset and name not in dataset.virtual_variables:
-            raise ValueError('name %r must be a variable in dataset %r'
-                             % (name, dataset))
         obj._dataset = dataset
         obj._name = name
         return obj
@@ -105,6 +224,23 @@ class DataArray(AbstractArray):
         """
         return self._dataset
 
+    def to_dataset(self, name=None):
+        """Convert a DataArray to a Dataset
+
+        Parameters
+        ----------
+        name : str, optional
+            Name to substitute for this array's name (if it has one).
+
+        Returns
+        -------
+        dataset : Dataset
+        """
+        if name is None:
+            return self._dataset.copy()
+        else:
+            return self.rename(name)._dataset
+
     @property
     def name(self):
         """The name of the variable in `dataset` to which array operations
@@ -112,19 +248,25 @@ class DataArray(AbstractArray):
         """
         return self._name
 
+    @contextlib.contextmanager
+    def _set_new_dataset(self):
+        """Context manager to use for modifying _dataset, in a manner that
+        can be safely rolled back if an error is encountered.
+        """
+        ds = self.dataset.copy(deep=False)
+        yield ds
+        self._dataset = ds
+
     @name.setter
     def name(self, value):
-        raise AttributeError('cannot modify the name of a %s inplace; use the '
-                             "'rename' method instead" % type(self).__name__)
+        with self._set_new_dataset() as ds:
+            ds.rename({self.name: value}, inplace=True)
+        self._name = value
 
     @property
     def variable(self):
         return self.dataset.variables[self.name]
 
-    @variable.setter
-    def variable(self, value):
-        self.dataset[self.name] = value
-
     @property
     def dtype(self):
         return self.variable.dtype
@@ -158,17 +300,38 @@ class DataArray(AbstractArray):
 
     @property
     def as_index(self):
-        """The variable's data as a pandas.Index. Only possible for 1D arrays.
+        utils.alias_warning('as_index', 'to_index()')
+        return self.to_index()
+
+    def to_index(self):
+        """Convert this variable to a pandas.Index. Only possible for 1D
+        arrays.
         """
-        return self.variable.to_coord().as_index
+        return self.variable.to_index()
+
+    @property
+    def dims(self):
+        return self.variable.dims
+
+    @dims.setter
+    def dims(self, value):
+        with self._set_new_dataset() as ds:
+            if not len(value) == self.ndim:
+                raise ValueError('%s dimensions supplied but data has ndim=%s'
+                                 % (len(value), self.ndim))
+            name_map = dict(zip(self.dims, value))
+            ds.rename(name_map, inplace=True)
+        if self.name in name_map:
+            self._name = name_map[self.name]
 
     @property
     def dimensions(self):
-        return self.variable.dimensions
+        utils.alias_warning('dimensions', 'dims')
+        return self.dims
 
     def _key_to_indexers(self, key):
         return OrderedDict(
-            zip(self.dimensions, indexing.expanded_indexer(key, self.ndim)))
+            zip(self.dims, indexing.expanded_indexer(key, self.ndim)))
 
     def __getitem__(self, key):
         if isinstance(key, basestring):
@@ -176,7 +339,7 @@ class DataArray(AbstractArray):
             return self.dataset[key]
         else:
             # orthogonal array indexing
-            return self.indexed(**self._key_to_indexers(key))
+            return self.isel(**self._key_to_indexers(key))
 
     def __setitem__(self, key, value):
         if isinstance(key, basestring):
@@ -200,12 +363,12 @@ class DataArray(AbstractArray):
 
     @property
     def attributes(self):
-        utils.alias_warning('attributes', 'attrs', 3)
+        utils.alias_warning('attributes', 'attrs')
         return self.variable.attrs
 
     @attributes.setter
     def attributes(self, value):
-        utils.alias_warning('attributes', 'attrs', 3)
+        utils.alias_warning('attributes', 'attrs')
         self.variable.attrs = value
 
     @property
@@ -223,12 +386,67 @@ class DataArray(AbstractArray):
         serialized."""
         return self.variable.encoding
 
+    @encoding.setter
+    def encoding(self, value):
+        self.variable.encoding = value
+
+    @property
+    def indexes(self):
+        return self.coords
+
+    @property
+    def coords(self):
+        """Dictionary-like container of xray.Coordinate objects used for label based
+        indexing.
+
+        Keys are given by the dimensions, but list-like (integer based)
+        indexing is also supported.
+        """
+        return DataArrayCoordinates(self)
+
+    @coords.setter
+    def coords(self, value):
+        if not len(value) == self.ndim:
+            raise ValueError('%s coordinates supplied but data has ndim=%s'
+                             % (len(value), self.ndim))
+        with self._set_new_dataset() as ds:
+            # TODO: allow setting to dict-like objects other than
+            # DataArrayCoordinates?
+            if isinstance(value, DataArrayCoordinates):
+                # yes, this is regretably complex and probably slow
+                name_map = dict(zip(self.dims, value.keys()))
+                ds.rename(name_map, inplace=True)
+                name = name_map.get(self.name, self.name)
+                dims = ds[name].dims
+                value = value.values()
+            else:
+                name = self.name
+                dims = self.dims
+
+            for k, v in zip(dims, value):
+                coord = DataArrayCoordinates._convert_to_coord(
+                    k, v, expected_size=ds.coords[k].size)
+                ds[k] = coord
+        self._name = name
+
     @property
     def coordinates(self):
-        """Dictionary of Coordinate objects used for label based indexing.
+        utils.alias_warning('coordinates', 'coords')
+        return self.coords
+
+    def load_data(self):
+        """Manually trigger loading of this array's data from disk or a
+        remote source into memory and return this array.
+
+        Normally, it should not be necessary to call this method in user code,
+        because all xray functions should either work on deferred data or
+        load data automatically. However, this method can be necessary when
+        working with many file objects on disk.
         """
-        return FrozenOrderedDict((dim, self.dataset.variables[dim])
-                                 for dim in self.dimensions)
+        self.variable.load_data()
+        for coord in self.coords.values():
+            coord.load_data()
+        return self
 
     def copy(self, deep=True):
         """Returns a copy of this array.
@@ -251,28 +469,32 @@ class DataArray(AbstractArray):
     # mutable objects should not be hashable
     __hash__ = None
 
-    def indexed(self, **indexers):
-        """Return a new DataArray whose dataset is given by indexing along
-        the specified dimension(s).
+    def isel(self, **indexers):
+        """Return a new DataArray whose dataset is given by integer indexing
+        along the specified dimension(s).
 
         See Also
         --------
-        Dataset.indexed
-        DataArray.labeled
+        Dataset.isel
+        DataArray.sel
         """
-        ds = self.dataset.indexed(**indexers)
+        ds = self.dataset.isel(**indexers)
         return ds[self.name]
 
-    def labeled(self, **indexers):
+    indexed = utils.function_alias(isel, 'indexed')
+
+    def sel(self, **indexers):
         """Return a new DataArray whose dataset is given by selecting
-        coordinate labels along the specified dimension(s).
+        index labels along the specified dimension(s).
 
         See Also
         --------
-        Dataset.labeled
-        DataArray.indexed
+        Dataset.sel
+        DataArray.isel
         """
-        return self.indexed(**indexing.remap_label_indexers(self, indexers))
+        return self.isel(**indexing.remap_label_indexers(self, indexers))
+
+    labeled = utils.function_alias(sel, 'labeled')
 
     def reindex_like(self, other, copy=True):
         """Conform this object onto the coordinates of another object, filling
@@ -280,13 +502,13 @@ class DataArray(AbstractArray):
 
         Parameters
         ----------
-        other : Dataset or DatasetArray
+        other : Dataset or DataArray
             Object with a coordinates attribute giving a mapping from dimension
-            names to xray.Variable objects, which provides coordinates upon
+            names to xray.Coordinate objects, which provides indexes upon
             which to index the variables in this dataset. The coordinates on
             this other object need not be the same as the coordinates on this
-            dataset. Any mis-matched coordinates values will be filled in with
-            NaN, and any mis-matched coordinate names will simply be ignored.
+            dataset. Any mis-matched coordinate values will be filled in with
+            NaN, and any mis-matched dimension names will simply be ignored.
         copy : bool, optional
             If `copy=True`, the returned array's dataset contains only copied
             variables. If `copy=False` and no reindexing is required then
@@ -294,20 +516,20 @@ class DataArray(AbstractArray):
 
         Returns
         -------
-        reindexed : DatasetArray
-            Another dataset array, with coordinates replaced from the other
-            object.
+        reindexed : DataArray
+            Another dataset array, with this array's data but coordinates from
+            the other object.
 
         See Also
         --------
-        DatasetArray.reindex
+        DataArray.reindex
         align
         """
-        return self.reindex(copy=copy, **other.coordinates)
+        return self.reindex(copy=copy, **other.coords)
 
-    def reindex(self, copy=True, **coordinates):
-        """Conform this object onto a new set of coordinates or pandas.Index
-        objects, filling in missing values with NaN.
+    def reindex(self, copy=True, **indexers):
+        """Conform this object onto a new set of coordinates, filling in
+        missing values with NaN.
 
         Parameters
         ----------
@@ -315,83 +537,93 @@ class DataArray(AbstractArray):
             If `copy=True`, the returned array's dataset contains only copied
             variables. If `copy=False` and no reindexing is required then
             original variables from this array's dataset are returned.
-        **coordinates : dict
+        **indexers : dict
             Dictionary with keys given by dimension names and values given by
-            arrays of coordinate labels. Any mis-matched coordinates values
-            will be filled in with NaN, and any mis-matched coordinate names
-            will simply be ignored.
+            arrays of coordinates tick labels. Any mis-matched coordinate values
+            will be filled in with NaN, and any mis-matched dimension names will
+            simply be ignored.
 
         Returns
         -------
-        reindexed : DatasetArray
-            Another dataset array, with replaced coordinates.
+        reindexed : DataArray
+            Another dataset array, with this array's data but replaced
+            coordinates.
 
         See Also
         --------
-        DatasetArray.reindex_like
+        DataArray.reindex_like
         align
         """
-        reindexed_ds = self.select().dataset.reindex(copy=copy, **coordinates)
+        ds = self.select_vars().dataset
+        reindexed_ds = ds.reindex(copy=copy, **indexers)
         return reindexed_ds[self.name]
 
     def rename(self, new_name_or_name_dict):
         """Returns a new DataArray with renamed variables.
 
-        If the argument is a string, rename this DataArray's arary variable.
-        Otherwise, the argument is assumed to be a mapping from old names to
-        new names for dataset variables.
+        If the argument is dict-like, it it used as a mapping from old names to
+        new names for dataset variables. Otherwise, use the argument as the new
+        name for this array.
 
         See Also
         --------
         Dataset.rename
         """
-        if isinstance(new_name_or_name_dict, basestring):
-            new_name = new_name_or_name_dict
-            name_dict = {self.name: new_name}
-        else:
+        if _is_dict_like(new_name_or_name_dict):
             name_dict = new_name_or_name_dict
             new_name = name_dict.get(self.name, self.name)
+        else:
+            new_name = new_name_or_name_dict
+            name_dict = {self.name: new_name}
         renamed_dataset = self.dataset.rename(name_dict)
         return renamed_dataset[new_name]
 
-    def select(self, *names):
+    def select_vars(self, *names):
         """Returns a new DataArray with only the named variables, as well
         as this DataArray's array variable (and all associated coordinates).
 
         See Also
         --------
-        Dataset.select
+        Dataset.select_vars
         """
         names = names + (self.name,)
-        ds = self.dataset.select(*names)
+        ds = self.dataset.select_vars(*names)
         return ds[self.name]
 
-    def unselect(self, *names):
+    select = utils.function_alias(select_vars, 'select')
+
+    def drop_vars(self, *names):
         """Returns a new DataArray without the named variables.
 
         See Also
         --------
-        Dataset.unselect
+        Dataset.drop_vars
         """
         if self.name in names:
-            raise ValueError('cannot unselect the name of a DataArray with '
-                             'unselect. Use the `unselect` method of the '
-                             'dataset instead.')
-        ds = self.dataset.unselect(*names)
+            raise ValueError('cannot drop the name of a DataArray with '
+                             'drop_vars. Use the `drop_vars` method of '
+                             'the dataset instead.')
+        if any(name in self.dims for name in names):
+            raise ValueError('cannot drop a coordinate variable from a '
+                             'DataArray. Use the `drop_vars` method of '
+                             'the dataset instead.')
+        ds = self.dataset.drop_vars(*names)
         return ds[self.name]
 
+    unselect = utils.function_alias(drop_vars, 'unselect')
+
     def groupby(self, group, squeeze=True):
         """Group this dataset by unique values of the indicated group.
 
         Parameters
         ----------
-        group : str or DataArray
+        group : str, DataArray or Coordinate
             Array whose unique values should be used to group this array. If a
             string, must be the name of a variable contained in this dataset.
         squeeze : boolean, optional
-            If "group" is a coordinate of this array, `squeeze` controls
+            If "group" is a diension of this array, `squeeze` controls
             whether the subarrays have a dimension of length 1 along that
-            coordinate or if the dimension is squeezed out.
+            dimension or if the dimension is squeezed out.
 
         Returns
         -------
@@ -403,10 +635,10 @@ class DataArray(AbstractArray):
             `std`, etc.).
         """
         if isinstance(group, basestring):
-            group = self[group]
+            group = self.dataset[group]
         return groupby.ArrayGroupBy(self, group, squeeze=squeeze)
 
-    def transpose(self, *dimensions):
+    def transpose(self, *dims):
         """Return a new DataArray object with transposed dimensions.
 
         Note: Although this operation returns a view of this array's data, it
@@ -414,7 +646,7 @@ class DataArray(AbstractArray):
 
         Parameters
         ----------
-        *dimensions : str, optional
+        *dims : str, optional
             By default, reverse the dimensions. Otherwise, reorder the
             dimensions to this order.
 
@@ -434,15 +666,15 @@ class DataArray(AbstractArray):
         Array.transpose
         """
         ds = self.dataset.copy()
-        ds[self.name] = self.variable.transpose(*dimensions)
+        ds[self.name] = self.variable.transpose(*dims)
         return ds[self.name]
 
-    def squeeze(self, dimension=None):
+    def squeeze(self, dim=None):
         """Return a new DataArray object with squeezed data.
 
         Parameters
         ----------
-        dimensions : None or str or tuple of str, optional
+        dim : None or str or tuple of str, optional
             Selects a subset of the length one dimensions. If a dimension is
             selected with length greater than one, an error is raised. If
             None, all length one dimensions are squeezed.
@@ -462,10 +694,10 @@ class DataArray(AbstractArray):
         --------
         numpy.squeeze
         """
-        ds = self.dataset.squeeze(dimension)
+        ds = self.dataset.squeeze(dim)
         return ds[self.name]
 
-    def reduce(self, func, dimension=None, axis=None, **kwargs):
+    def reduce(self, func, dim=None, axis=None, keep_attrs=False, **kwargs):
         """Reduce this array by applying `func` along some dimension(s).
 
         Parameters
@@ -474,13 +706,17 @@ class DataArray(AbstractArray):
             Function which can be called in the form
             `f(x, axis=axis, **kwargs)` to return the result of reducing an
             np.ndarray over an integer valued axis.
-        dimension : str or sequence of str, optional
+        dim : str or sequence of str, optional
             Dimension(s) over which to apply `func`.
         axis : int or sequence of int, optional
             Axis(es) over which to repeatedly apply `func`. Only one of the
-            'dimension' and 'axis' arguments can be supplied. If neither are
+            'dim' and 'axis' arguments can be supplied. If neither are
             supplied, then the reduction is calculated over the flattened array
             (by calling `f(x)` without an axis argument).
+        keep_attrs : bool, optional
+            If True, the variable's attributes (`attrs`) will be copied from
+            the original object to the new one.  If False (default), the new
+            object will be returned without attributes.
         **kwargs : dict
             Additional keyword arguments passed on to `func`.
 
@@ -490,19 +726,27 @@ class DataArray(AbstractArray):
             DataArray with this object's array replaced with an array with
             summarized data and the indicated dimension(s) removed.
         """
-        var = self.variable.reduce(func, dimension, axis, **kwargs)
-        drop = set(self.dimensions) - set(var.dimensions)
+        if 'dimension' in kwargs and dim is None:
+            dim = kwargs.pop('dimension')
+            utils.alias_warning('dimension', 'dim')
+
+        var = self.variable.reduce(func, dim, axis, keep_attrs, **kwargs)
+        drop = set(self.dims) - set(var.dims)
         # For now, take an aggressive strategy of removing all variables
         # associated with any dropped dimensions
         # TODO: save some summary (mean? bounds?) of dropped variables
-        drop |= {k for k, v in iteritems(self.dataset.variables)
-                 if any(dim in drop for dim in v.dimensions)}
-        ds = self.dataset.unselect(*drop)
+        drop |= set(k for k, v in iteritems(self.dataset.variables)
+                    if any(dim in drop for dim in v.dims))
+        ds = self.dataset.drop_vars(*drop)
         ds[self.name] = var
+
+        if keep_attrs:
+            ds.attrs = self.dataset.attrs
+
         return ds[self.name]
 
     @classmethod
-    def concat(cls, arrays, dimension='concat_dimension', indexers=None,
+    def concat(cls, arrays, dim='concat_dim', indexers=None,
                concat_over=None):
         """Stack arrays along a new or existing dimension to form a new
         DataArray.
@@ -513,7 +757,7 @@ class DataArray(AbstractArray):
             Arrays to stack together. Each variable is expected to have
             matching dimensions and shape except for along the concatenated
             dimension.
-        dimension : str or Array, optional
+        dim : str or Array, optional
             Name of the dimension to stack along. This can either be a new
             dimension name, in which case it is added along axis=0, or an
             existing dimension name, in which case the location of the
@@ -554,9 +798,11 @@ class DataArray(AbstractArray):
             datasets.append(arr.dataset)
         if concat_over is None:
             concat_over = set()
-        concat_over = set(concat_over) | {name}
-        ds = xray.Dataset.concat(datasets, dimension, indexers,
-                                     concat_over=concat_over)
+        elif isinstance(concat_over, basestring):
+            concat_over = set([concat_over])
+        concat_over = set(concat_over) | set([name])
+        ds = xray.Dataset.concat(datasets, dim, indexers,
+                                 concat_over=concat_over)
         return ds[name]
 
     def to_dataframe(self):
@@ -564,7 +810,7 @@ class DataArray(AbstractArray):
 
         Non-coordinate variables in this array's dataset (which include this
         array's data) form the columns of the DataFrame. The DataFrame is be
-        indexed by the Cartesian product of the dataset's coordinates.
+        indexed by the Cartesian product of the dataset's coordintaes.
         """
         return self.dataset.to_dataframe()
 
@@ -573,25 +819,24 @@ class DataArray(AbstractArray):
 
         The Series is indexed by the Cartesian product of the coordinates.
         Unlike `to_dataframe`, only this array is including in the returned
-        series; the other non-coordinates variables in the dataset are not.
+        series; the other non-coordinate variables in the dataset are not.
         """
-        index = multi_index_from_product(self.coordinates.values(),
-                                         names=self.coordinates.keys())
+        index = multi_index_from_product(self.coords.values(),
+                                         names=self.coords.keys())
         return pd.Series(self.values.reshape(-1), index=index, name=self.name)
 
     @classmethod
     def from_series(cls, series):
-        """Convert a pandas.Series into an xray.DatasetArray
+        """Convert a pandas.Series into an xray.DataArray
 
         If the series's index is a MultiIndex, it will be expanded into a
-        tensor product of one-dimensional indices  (filling in missing values
+        tensor product of one-dimensional coordinates (filling in missing values
         with NaN). Thus this operation should be the inverse of the `to_series`
         method.
         """
-        name = series.name if series.name is not None else 'values'
-        df = pd.DataFrame({name: series})
+        df = pd.DataFrame({series.name: series})
         ds = xray.Dataset.from_dataframe(df)
-        return ds[name]
+        return ds[series.name]
 
     def equals(self, other):
         """True if two DataArrays have the same dimensions, coordinates and
@@ -606,8 +851,8 @@ class DataArray(AbstractArray):
         try:
             return (all(k1 == k2 and v1.equals(v2)
                         for (k1, v1), (k2, v2)
-                        in zip(self.coordinates.items(),
-                               other.coordinates.items()))
+                        in zip(self.coords.items(),
+                               other.coords.items()))
                     and self.variable.equals(other.variable))
         except AttributeError:
             return False
@@ -620,46 +865,39 @@ class DataArray(AbstractArray):
             return (self.name == other.name
                     and all(k1 == k2 and v1.identical(v2)
                             for (k1, v1), (k2, v2)
-                            in zip(self.coordinates.items(),
-                                   other.coordinates.items()))
+                            in zip(self.coords.items(),
+                                   other.coords.items()))
                     and self.variable.identical(other.variable))
         except AttributeError:
             return False
 
-    def _select_coordinates(self):
-        return xray.Dataset(self.coordinates)
-
-    def _refocus(self, new_var, name=None):
-        """Returns a copy of this DataArray's dataset with this
-        DataArray's focus variable replaced by `new_var`.
-
-        If `new_var` is a DataArray, its contents will be merged in.
-        """
-        if not hasattr(new_var, 'dimensions'):
-            new_var = variable.Variable(self.variable.dimensions, new_var)
-        ds = self._select_coordinates()
-        if name is None:
-            name = self.name + '_'
-        ds[name] = new_var
-        return ds[name]
+    def _select_coords(self):
+        return xray.Dataset(self.coords)
 
     def __array_wrap__(self, obj, context=None):
-        return self._refocus(self.variable.__array_wrap__(obj, context))
+        new_var = self.variable.__array_wrap__(obj, context)
+        ds = self._select_coords()
+        if (self.name,) == self.dims:
+            # use a new name for coordinate variables
+            name = None
+        else:
+            name = self.name
+        ds[name] = new_var
+        return ds[name]
 
     @staticmethod
     def _unary_op(f):
         @functools.wraps(f)
         def func(self, *args, **kwargs):
-            return self._refocus(f(self.variable, *args, **kwargs),
-                                 self.name + '_' + f.__name__)
+            return self.__array_wrap__(f(self.values, *args, **kwargs))
         return func
 
-    def _check_coordinates_compat(self, other):
+    def _check_coords_compat(self, other):
         # TODO: possibly automatically select index intersection instead?
-        if hasattr(other, 'coordinates'):
-            for k, v in iteritems(self.coordinates):
-                if (k in other.coordinates
-                        and not v.equals(other.coordinates[k])):
+        if hasattr(other, 'coords'):
+            for k, v in iteritems(self.coords):
+                if (k in other.coords
+                        and not v.equals(other.coords[k])):
                     raise ValueError('coordinate %r is not aligned' % k)
 
     @staticmethod
@@ -668,16 +906,18 @@ class DataArray(AbstractArray):
         def func(self, other):
             # TODO: automatically group by other variable dimensions to allow
             # for broadcasting dimensions like 'dayofyear' against 'time'
-            self._check_coordinates_compat(other)
-            ds = self._select_coordinates()
-            if hasattr(other, 'coordinates'):
-                ds.merge(other.coordinates, inplace=True)
+            self._check_coords_compat(other)
+            ds = self._select_coords()
+            if hasattr(other, 'coords'):
+                ds.merge(other.coords, inplace=True)
             other_array = getattr(other, 'variable', other)
-            other_name = getattr(other, 'name', 'other')
-            name = self.name + '_' + f.__name__ + '_' + other_name
+            if hasattr(other, 'name') or (self.name,) == self.dims:
+                name = None
+            else:
+                name = self.name
             ds[name] = (f(self.variable, other_array)
-                         if not reflexive
-                         else f(other_array, self.variable))
+                        if not reflexive
+                        else f(other_array, self.variable))
             return ds[name]
         return func
 
@@ -685,11 +925,11 @@ class DataArray(AbstractArray):
     def _inplace_binary_op(f):
         @functools.wraps(f)
         def func(self, other):
-            self._check_coordinates_compat(other)
+            self._check_coords_compat(other)
             other_array = getattr(other, 'variable', other)
-            self.variable = f(self.variable, other_array)
-            if hasattr(other, 'coordinates'):
-                self.dataset.merge(other.coordinates, inplace=True)
+            f(self.variable, other_array)
+            if hasattr(other, 'coords'):
+                self.dataset.merge(other.coords, inplace=True)
             return self
         return func
 
@@ -703,13 +943,14 @@ def align(*objects, **kwargs):
     objects with aligned coordinates.
 
     Array from the aligned objects are suitable as input to mathematical
-    operators, because along each dimension they have the same coordinates.
+    operators, because along each dimension they are indexed by the same
+    coordinates.
 
     Missing values (if ``join != 'inner'``) are filled with NaN.
 
     Parameters
     ----------
-    *objects : Dataset or DatasetArray
+    *objects : Dataset or DataArray
         Objects to align.
     join : {'outer', 'inner', 'left', 'right'}, optional
         Method for joining the coordinates of the passed objects along each
@@ -749,14 +990,14 @@ def align(*objects, **kwargs):
     elif join == 'right':
         join_indices = operator.itemgetter(-1)
 
-    all_coords = defaultdict(list)
+    all_indexes = defaultdict(list)
     for obj in objects:
-        for k, v in iteritems(obj.coordinates):
-            all_coords[k].append(v.as_index)
+        for k, v in iteritems(obj.coords):
+            all_indexes[k].append(v.to_index())
 
     # Exclude dimensions with all equal indices to avoid unnecessary reindexing
     # work.
-    joined_coords = {k: join_indices(v) for k, v in iteritems(all_coords)
-                     if any(not v[0].equals(idx) for idx in v[1:])}
+    joined_indexes = dict((k, join_indices(v)) for k, v in iteritems(all_indexes)
+                          if any(not v[0].equals(idx) for idx in v[1:]))
 
-    return tuple(obj.reindex(copy=copy, **joined_coords) for obj in objects)
+    return tuple(obj.reindex(copy=copy, **joined_indexes) for obj in objects)
