@@ -6,12 +6,10 @@ try:  # Python 2
     from itertools import izip
 except ImportError: # Python 3
     izip = zip
-from collections import OrderedDict
 
-from . import groupby
 from . import indexing
 from . import ops
-from .pycompat import basestring
+from .pycompat import basestring, OrderedDict
 from . import utils
 import xray
 
@@ -24,7 +22,7 @@ def as_variable(obj, strict=True):
     - If the object is already an `Variable`, return it.
     - If the object is a `DataArray`, return it if `strict=False` or return
       its variable if `strict=True`.
-    - Otherwise, if the object has 'dimensions' and 'data' attributes, convert
+    - Otherwise, if the object has 'dims' and 'data' attributes, convert
       it into a new `Variable`.
     - If all else fails, attempt to convert the object into an `Variable` by
       unpacking it into the arguments for `Variable.__init__`.
@@ -35,8 +33,8 @@ def as_variable(obj, strict=True):
         # extract the primary Variable from DataArrays
         obj = obj.variable
     if not isinstance(obj, (Variable, xray.DataArray)):
-        if hasattr(obj, 'dimensions') and hasattr(obj, 'values'):
-            obj = Variable(obj.dimensions, obj.values,
+        if hasattr(obj, 'dims') and hasattr(obj, 'values'):
+            obj = Variable(obj.dims, obj.values,
                            getattr(obj, 'attributes', None),
                            getattr(obj, 'encoding', None))
         else:
@@ -44,6 +42,9 @@ def as_variable(obj, strict=True):
                 raise TypeError('cannot convert numpy.ndarray objects into '
                                 'Variable objects without supplying '
                                 'dimensions')
+            if not isinstance(obj, tuple):
+                raise TypeError('can only convert tuples into parameters for '
+                                'xray.Variable parameters')
             try:
                 obj = Variable(*obj)
             except TypeError:
@@ -69,7 +70,8 @@ def _as_compatible_data(data):
     # don't check for __len__ or __iter__ so as not to cast if data is a numpy
     # numeric type like np.float32
     required = ['dtype', 'shape', 'size', 'ndim']
-    if any(not hasattr(data, attr) for attr in required):
+    if (any(not hasattr(data, attr) for attr in required)
+            or isinstance(data, np.string_)):
         # data must be ndarray-like
         data = np.asarray(data)
     elif isinstance(data, np.datetime64):
@@ -192,7 +194,7 @@ class Variable(AbstractArray):
     described outside the context of its parent Dataset (if you want such a
     fully described object, use a DataArray instead).
     """
-    def __init__(self, dims, data, attributes=None, encoding=None):
+    def __init__(self, dims, data, attrs=None, encoding=None):
         """
         Parameters
         ----------
@@ -202,7 +204,7 @@ class Variable(AbstractArray):
             number of dimensions.
         data : array_like
             Data array which supports numpy-like data access.
-        attributes : dict_like or None, optional
+        attrs : dict_like or None, optional
             Attributes to assign to the new variable. If None (default), an
             empty attribute dictionary is initialized.
         encoding : dict_like or None, optional
@@ -213,11 +215,11 @@ class Variable(AbstractArray):
             unrecognized encoding items.
         """
         self._data = _as_compatible_data(data)
-        self._dimensions = self._parse_dimensions(dims)
-        if attributes is None:
-            attributes = {}
-        self._attributes = OrderedDict(attributes)
-        self.encoding = dict({} if encoding is None else encoding)
+        self._dims = self._parse_dimensions(dims)
+        if attrs is None:
+            attrs = {}
+        self._attrs = OrderedDict(attrs)
+        self._encoding = dict({} if encoding is None else encoding)
 
     @property
     def dtype(self):
@@ -248,6 +250,17 @@ class Variable(AbstractArray):
             self._data = self._cache_data_class(self._data)
         return self._data
 
+    def load_data(self):
+        """Manually trigger loading of this variable's data from disk or a
+        remote source into memory and return this variable.
+
+        Normally, it should not be necessary to call this method in user code,
+        because all xray functions should either work on deferred data or
+        load data automatically.
+        """
+        self._data_cached()
+        return self
+
     def __getstate__(self):
         """Always cache data as an in-memory array before pickling"""
         self._data_cached()
@@ -269,15 +282,29 @@ class Variable(AbstractArray):
         self._data = values
 
     def to_coord(self):
-        """Return this variable as a Coordinate"""
-        return Coordinate(self.dimensions, self._data, self.attrs,
+        """Return this variable as an xray.Coordinate"""
+        return Coordinate(self.dims, self._data, self.attrs,
                           encoding=self.encoding)
 
     @property
-    def dimensions(self):
+    def as_index(self):
+        utils.alias_warning('as_index', 'to_index()')
+        return self.to_index()
+
+    def to_index(self):
+        """Convert this variable to a pandas.Index"""
+        return self.to_coord().to_index()
+
+    @property
+    def dims(self):
         """Tuple of dimension names with which this variable is associated.
         """
-        return self._dimensions
+        return self._dims
+
+    @property
+    def dimensions(self):
+        utils.alias_warning('dimensions', 'dims')
+        return self.dims
 
     def _parse_dimensions(self, dims):
         if isinstance(dims, basestring):
@@ -289,9 +316,9 @@ class Variable(AbstractArray):
                              % (dims, self.ndim))
         return dims
 
-    @dimensions.setter
-    def dimensions(self, value):
-        self._dimensions = self._parse_dimensions(value)
+    @dims.setter
+    def dims(self, value):
+        self._dims = self._parse_dimensions(value)
 
     def __getitem__(self, key):
         """Return a new Array object whose contents are consistent with
@@ -302,24 +329,24 @@ class Variable(AbstractArray):
         (including `Ellipsis`) and 1d arrays, each of which are applied
         orthogonally along their respective dimensions.
 
-        The difference not matter in most cases unless you are using numpy's
-        "fancy indexing," which can otherwise result in data arrays
-        with shapes is inconsistent (or just uninterpretable with) with the
+        The difference does not matter in most cases unless you are using
+        numpy's "fancy indexing," which can otherwise result in data arrays
+        whose shapes is inconsistent (or just uninterpretable with) with the
         variable's dimensions.
 
         If you really want to do indexing like `x[x > 0]`, manipulate the numpy
         array `x.values` directly.
         """
         key = indexing.expanded_indexer(key, self.ndim)
-        dimensions = [dim for k, dim in zip(key, self.dimensions)
-                        if not isinstance(k, (int, np.integer))]
+        dims = [dim for k, dim in zip(key, self.dims)
+                      if not isinstance(k, (int, np.integer))]
         values = self._data[key]
         # orthogonal indexing should ensure the dimensionality is consistent
         if hasattr(values, 'ndim'):
-            assert values.ndim == len(dimensions), (values.ndim, len(dimensions))
+            assert values.ndim == len(dims), (values.ndim, len(dims))
         else:
-            assert len(dimensions) == 0, len(dimensions)
-        return type(self)(dimensions, values, self.attrs, self.encoding)
+            assert len(dims) == 0, len(dims)
+        return type(self)(dims, values, self.attrs)
 
     def __setitem__(self, key, value):
         """__setitem__ is overloaded to access the underlying numpy values with
@@ -343,11 +370,21 @@ class Variable(AbstractArray):
     def attrs(self):
         """Dictionary of local attributes on this variable.
         """
-        return self._attributes
+        return self._attrs
 
     @attrs.setter
     def attrs(self, value):
-        self._attributes = OrderedDict(value)
+        self._attrs = OrderedDict(value)
+
+    @property
+    def encoding(self):
+        """Dictionary of encodings on this variable.
+        """
+        return self._encoding
+
+    @encoding.setter
+    def encoding(self, value):
+        self._encoding = dict(value)
 
     def copy(self, deep=True):
         """Returns a copy of this object.
@@ -357,9 +394,9 @@ class Variable(AbstractArray):
         """
         data = self.values.copy() if deep else self._data
         # note:
-        # dimensions is already an immutable tuple
+        # dims is already an immutable tuple
         # attributes and encoding will be copied when the new Array is created
-        return type(self)(self.dimensions, data, self.attrs, self.encoding)
+        return type(self)(self.dims, data, self.attrs, self.encoding)
 
     def __copy__(self):
         return self.copy(deep=False)
@@ -372,7 +409,7 @@ class Variable(AbstractArray):
     # mutable objects should not be hashable
     __hash__ = None
 
-    def indexed(self, **indexers):
+    def isel(self, **indexers):
         """Return a new array indexed along the specified dimension(s).
 
         Parameters
@@ -389,22 +426,24 @@ class Variable(AbstractArray):
             unless numpy fancy indexing was triggered by using an array
             indexer, in which case the data will be a copy.
         """
-        invalid = [k for k in indexers if not k in self.dimensions]
+        invalid = [k for k in indexers if not k in self.dims]
         if invalid:
             raise ValueError("dimensions %r do not exist" % invalid)
 
         key = [slice(None)] * self.ndim
-        for i, dim in enumerate(self.dimensions):
+        for i, dim in enumerate(self.dims):
             if dim in indexers:
                 key[i] = indexers[dim]
         return self[tuple(key)]
 
-    def transpose(self, *dimensions):
+    indexed = utils.function_alias(isel, 'indexed')
+
+    def transpose(self, *dims):
         """Return a new Variable object with transposed dimensions.
 
         Parameters
         ----------
-        *dimensions : str, optional
+        *dims : str, optional
             By default, reverse the dimensions. Otherwise, reorder the
             dimensions to this order.
 
@@ -423,18 +462,18 @@ class Variable(AbstractArray):
         --------
         numpy.transpose
         """
-        if len(dimensions) == 0:
-            dimensions = self.dimensions[::-1]
-        axes = self.get_axis_num(dimensions)
+        if len(dims) == 0:
+            dims = self.dims[::-1]
+        axes = self.get_axis_num(dims)
         data = self.values.transpose(*axes)
-        return type(self)(dimensions, data, self.attrs, self.encoding)
+        return type(self)(dims, data, self.attrs, self.encoding)
 
-    def squeeze(self, dimension=None):
+    def squeeze(self, dim=None):
         """Return a new Variable object with squeezed data.
 
         Parameters
         ----------
-        dimensions : None or str or tuple of str, optional
+        dim : None or str or tuple of str, optional
             Selects a subset of the length one dimensions. If a dimension is
             selected with length greater than one, an error is raised. If
             None, all length one dimensions are squeezed.
@@ -454,10 +493,11 @@ class Variable(AbstractArray):
         --------
         numpy.squeeze
         """
-        dimensions = dict(zip(self.dimensions, self.shape))
-        return utils.squeeze(self, dimensions, dimension)
+        dims = dict(zip(self.dims, self.shape))
+        return utils.squeeze(self, dims, dim)
 
-    def reduce(self, func, dimension=None, axis=None, **kwargs):
+    def reduce(self, func, dim=None, axis=None, keep_attrs=False,
+               **kwargs):
         """Reduce this array by applying `func` along some dimension(s).
 
         Parameters
@@ -466,13 +506,17 @@ class Variable(AbstractArray):
             Function which can be called in the form
             `func(x, axis=axis, **kwargs)` to return the result of reducing an
             np.ndarray over an integer valued axis.
-        dimension : str or sequence of str, optional
+        dim : str or sequence of str, optional
             Dimension(s) over which to apply `func`.
         axis : int or sequence of int, optional
-            Axis(es) over which to apply `func`. Only one of the 'dimension'
+            Axis(es) over which to apply `func`. Only one of the 'dim'
             and 'axis' arguments can be supplied. If neither are supplied, then
             the reduction is calculated over the flattened array (by calling
             `func(x)` without an axis argument).
+        keep_attrs : bool, optional
+            If True, the variable's attributes (`attrs`) will be copied from
+            the original object to the new one.  If False (default), the new
+            object will be returned without attributes.
         **kwargs : dict
             Additional keyword arguments passed on to `func`.
 
@@ -482,24 +526,29 @@ class Variable(AbstractArray):
             Array with summarized data and the indicated dimension(s)
             removed.
         """
-        if dimension is not None and axis is not None:
-            raise ValueError("cannot supply both 'axis' and 'dimension' "
-                             "arguments")
+        if 'dimension' in kwargs and dim is None:
+            dim = kwargs.pop('dimension')
+            utils.alias_warning('dimension', 'dim')
 
-        if dimension is not None:
-            axis = self.get_axis_num(dimension)
+        if dim is not None and axis is not None:
+            raise ValueError("cannot supply both 'axis' and 'dim' arguments")
+
+        if dim is not None:
+            axis = self.get_axis_num(dim)
         data = func(self.values, axis=axis, **kwargs)
 
         removed_axes = (range(self.ndim) if axis is None
                         else np.atleast_1d(axis) % self.ndim)
-        dims = [dim for n, dim in enumerate(self.dimensions)
+        dims = [dim for n, dim in enumerate(self.dims)
                 if n not in removed_axes]
 
-        return Variable(dims, data)
+        attrs = self.attrs if keep_attrs else {}
+
+        return Variable(dims, data, attrs=attrs)
 
     @classmethod
-    def concat(cls, variables, dimension='stacked_dimension',
-               indexers=None, length=None, shortcut=False):
+    def concat(cls, variables, dim='concat_dim', indexers=None, length=None,
+               shortcut=False):
         """Concatenate variables along a new or existing dimension.
 
         Parameters
@@ -508,7 +557,7 @@ class Variable(AbstractArray):
             Arrays to stack together. Each variable is expected to have
             matching dimensions and shape except for along the stacked
             dimension.
-        dimension : str or DataArray, optional
+        dim : str or DataArray, optional
             Name of the dimension to stack along. This can either be a new
             dimension name, in which case it is added along axis=0, or an
             existing dimension name, in which case the location of the
@@ -537,17 +586,17 @@ class Variable(AbstractArray):
             Concatenated Variable formed by stacking all the supplied variables
             along the given dimension.
         """
-        if not isinstance(dimension, basestring):
-            length = dimension.size
-            dimension, = dimension.dimensions
+        if not isinstance(dim, basestring):
+            length = dim.size
+            dim, = dim.dims
 
         if length is None or indexers is None:
             # so much for lazy evaluation! we need to look at all the variables
             # to figure out the indexers and/or dimensions of the stacked
             # variable
             variables = list(variables)
-            steps = [var.shape[var.get_axis_num(dimension)]
-                     if dimension in var.dimensions else 1
+            steps = [var.shape[var.get_axis_num(dim)]
+                     if dim in var.dims else 1
                      for var in variables]
             if length is None:
                 length = sum(steps)
@@ -560,34 +609,35 @@ class Variable(AbstractArray):
                 if i != length:
                     raise ValueError('actual length of stacked variables '
                                      'along %s is %r but expected length was '
-                                     '%s' % (dimension, i, length))
+                                     '%s' % (dim, i, length))
 
         # initialize the stacked variable with empty data
+        from . import groupby
         first_var, variables = groupby.peek_at(variables)
-        if dimension in first_var.dimensions:
-            axis = first_var.get_axis_num(dimension)
+        if dim in first_var.dims:
+            axis = first_var.get_axis_num(dim)
             shape = tuple(length if n == axis else s
                           for n, s in enumerate(first_var.shape))
-            dims = first_var.dimensions
+            dims = first_var.dims
         else:
             axis = 0
             shape = (length,) + first_var.shape
-            dims = (dimension,) + first_var.dimensions
+            dims = (dim,) + first_var.dims
 
         concatenated = cls(dims, np.empty(shape, dtype=first_var.dtype))
         concatenated.attrs.update(first_var.attrs)
 
-        alt_dims = tuple(d for d in dims if d != dimension)
+        alt_dims = tuple(d for d in dims if d != dim)
 
         # copy in the data from the variables
         for var, indexer in izip(variables, indexers):
             if not shortcut:
                 # do sanity checks & attributes clean-up
-                if dimension in var.dimensions:
-                    # transpose verifies that the dimensions are equivalent
-                    if var.dimensions != concatenated.dimensions:
-                        var = var.transpose(*concatenated.dimensions)
-                elif var.dimensions != alt_dims:
+                if dim in var.dims:
+                    # transpose verifies that the dims are equivalent
+                    if var.dims != concatenated.dims:
+                        var = var.transpose(*concatenated.dims)
+                elif var.dims != alt_dims:
                     raise ValueError('inconsistent dimensions')
                 utils.remove_incompatible_items(concatenated.attrs, var.attrs)
 
@@ -599,9 +649,6 @@ class Variable(AbstractArray):
 
     def _data_equals(self, other):
         return (self._data is other._data
-                or ((not isinstance(self.values, np.ndarray)
-                     or not isinstance(other.values, np.ndarray))
-                     and self.values == other.values)
                 or utils.array_equiv(self.values, other.values))
 
     def equals(self, other):
@@ -616,7 +663,7 @@ class Variable(AbstractArray):
         """
         other = getattr(other, 'variable', other)
         try:
-            return (self.dimensions == other.dimensions
+            return (self.dims == other.dims
                     and self._data_equals(other))
         except (TypeError, AttributeError):
             return False
@@ -625,19 +672,19 @@ class Variable(AbstractArray):
         """Like equals, but also checks attributes.
         """
         try:
-            return (utils.dict_equal(self.attrs, other.attrs)
+            return (utils.dict_equiv(self.attrs, other.attrs)
                     and self.equals(other))
         except (TypeError, AttributeError):
             return False
 
     def __array_wrap__(self, obj, context=None):
-        return Variable(self.dimensions, obj)
+        return Variable(self.dims, obj)
 
     @staticmethod
     def _unary_op(f):
         @functools.wraps(f)
         def func(self, *args, **kwargs):
-            return Variable(self.dimensions, f(self.values, *args, **kwargs))
+            return Variable(self.dims, f(self.values, *args, **kwargs))
         return func
 
     @staticmethod
@@ -658,7 +705,7 @@ class Variable(AbstractArray):
         @functools.wraps(f)
         def func(self, other):
             self_data, other_data, dims = _broadcast_variable_data(self, other)
-            if dims != self.dimensions:
+            if dims != self.dims:
                 raise ValueError('dimensions cannot change for in-place '
                                  'operations')
             self.values = f(self_data, other_data)
@@ -669,16 +716,25 @@ ops.inject_special_operations(Variable)
 
 
 class Coordinate(Variable):
-    """Subclass of Variable which caches its data as a pandas.Index instead of
-    a numpy.ndarray.
+    """Wrapper around pandas.Index that adds xray specific functionality.
 
-    Coordinates must always be 1-dimensional. In addition to Variable methods,
-    they support some pandas.Index methods directly (e.g., get_indexer).
+    The most important difference is that Coordinate objects must always have a
+    name, which is the dimension along which they index values.
+
+    Coordinates must always be 1-dimensional. In addition to Variable methods
+    and properties (attributes, encoding, broadcasting), they support some
+    pandas.Index methods directly (e.g., get_indexer), even though pandas does
+    not (yet) support duck-typing for indexes.
     """
     _cache_data_class = PandasIndexAdapter
 
-    def __init__(self, *args, **kwargs):
-        super(Coordinate, self).__init__(*args, **kwargs)
+    def __init__(self, name, data, attrs=None, encoding=None):
+        if isinstance(data, pd.MultiIndex):
+            raise NotImplementedError(
+                'no support yet for using a pandas.MultiIndex in an '
+                'xray.Coordinate')
+
+        super(Coordinate, self).__init__(name, data, attrs, encoding)
         if self.ndim != 1:
             raise ValueError('%s objects must be 1-dimensional' %
                              type(self).__name__)
@@ -688,8 +744,7 @@ class Coordinate(Variable):
         if not hasattr(values, 'ndim') or values.ndim == 0:
             return Variable((), values, self.attrs, self.encoding)
         else:
-            return type(self)(self.dimensions, values, self.attrs,
-                              self.encoding)
+            return type(self)(self.dims, values, self.attrs, self.encoding)
 
     def __setitem__(self, key, value):
         raise TypeError('%s values cannot be modified' % type(self).__name__)
@@ -703,38 +758,50 @@ class Coordinate(Variable):
         # there is no need to copy the index values here even if deep=True
         # since pandas.Index objects are immutable
         data = PandasIndexAdapter(self) if deep else self._data
-        return type(self)(self.dimensions, data, self.attrs, self.encoding)
-
-    @property
-    def as_index(self):
-        """The variable's data as a pandas.Index"""
-        return self._data_cached().array
+        return type(self)(self.dims, data, self.attrs, self.encoding)
 
     def _data_equals(self, other):
-        return self.as_index.equals(other.to_coord().as_index)
+        return self.to_index().equals(other.to_index())
 
     def to_coord(self):
-        """Return this variable as an Coordinate"""
+        """Return this variable as an xray.Coordinate"""
         return self
 
+    def to_index(self):
+        """Convert this variable to a pandas.Index"""
+        # n.b. creating a new pandas.Index from an old pandas.Index is
+        # basically free as pandas.Index objects are immutable
+        assert self.ndim == 1
+        return pd.Index(self._data_cached().array, name=self.dims[0])
+
+    # pandas.Index like properties:
+
+    @property
+    def name(self):
+        return self.dims[0]
+
+    @name.setter
+    def name(self, value):
+        raise AttributeError('cannot modify name of Coordinate in-place')
+
     def get_indexer(self, label):
-        return self.as_index.get_indexer(label)
+        return self.to_index().get_indexer(label)
 
     def slice_indexer(self, start=None, stop=None, step=None):
-        return self.as_index.slice_indexer(start, stop, step)
+        return self.to_index().slice_indexer(start, stop, step)
 
     def slice_locs(self, start=None, stop=None):
-        return self.as_index.slice_locs(start, stop)
+        return self.to_index().slice_locs(start, stop)
 
     def get_loc(self, label):
-        return self.as_index.get_loc(label)
+        return self.to_index().get_loc(label)
 
     @property
     def is_monotonic(self):
-        return self.as_index.is_monotonic
+        return self.to_index().is_monotonic
 
     def is_numeric(self):
-        return self.as_index.is_numeric()
+        return self.to_index().is_numeric()
 
 
 def broadcast_variables(first, second):
@@ -758,33 +825,32 @@ def broadcast_variables(first, second):
     """
     # TODO: add unit tests specifically for this function
     # validate dimensions
-    dim_lengths = dict(zip(first.dimensions, first.shape))
-    for k, v in zip(second.dimensions, second.shape):
+    dim_lengths = dict(zip(first.dims, first.shape))
+    for k, v in zip(second.dims, second.shape):
         if k in dim_lengths and dim_lengths[k] != v:
             raise ValueError('operands could not be broadcast together '
                              'with mismatched lengths for dimension %r: %s'
                              % (k, (dim_lengths[k], v)))
-    for dimensions in [first.dimensions, second.dimensions]:
-        if len(set(dimensions)) < len(dimensions):
+    for dims in [first.dims, second.dims]:
+        if len(set(dims)) < len(dims):
             raise ValueError('broadcasting requires that neither operand '
-                             'has duplicate dimensions: %r'
-                             % list(dimensions))
+                             'has duplicate dimensions: %r' % list(dims))
 
     # build dimensions for new Array
-    second_only_dims = [d for d in second.dimensions
-                        if d not in first.dimensions]
-    dimensions = list(first.dimensions) + second_only_dims
+    second_only_dims = [d for d in second.dims
+                        if d not in first.dims]
+    dims = list(first.dims) + second_only_dims
 
     # expand first_data's dimensions so it's broadcast compatible after
     # adding second's dimensions at the end
     first_data = first.values[(Ellipsis,) + (None,) * len(second_only_dims)]
-    new_first = Variable(dimensions, first_data, first.attrs, first.encoding)
+    new_first = Variable(dims, first_data, first.attrs, first.encoding)
     # expand and reorder second_data so the dimensions line up
-    first_only_dims = [d for d in dimensions if d not in second.dimensions]
-    second_dims = list(second.dimensions) + first_only_dims
+    first_only_dims = [d for d in dims if d not in second.dims]
+    second_dims = list(second.dims) + first_only_dims
     second_data = second.values[(Ellipsis,) + (None,) * len(first_only_dims)]
     new_second = Variable(second_dims, second_data, second.attrs,
-                          second.encoding).transpose(*dimensions)
+                          second.encoding).transpose(*dims)
     return new_first, new_second
 
 
@@ -792,15 +858,15 @@ def _broadcast_variable_data(self, other):
     if isinstance(other, xray.Dataset):
         raise TypeError('datasets do not support mathematical operations')
     elif all(hasattr(other, attr) for attr
-             in ['dimensions', 'values', 'shape', 'encoding']):
+             in ['dims', 'values', 'shape', 'encoding']):
         # `other` satisfies the necessary Variable API for broadcast_variables
         new_self, new_other = broadcast_variables(self, other)
         self_data = new_self.values
         other_data = new_other.values
-        dimensions = new_self.dimensions
+        dims = new_self.dims
     else:
         # rely on numpy broadcasting rules
         self_data = self.values
         other_data = other
-        dimensions = self.dimensions
-    return self_data, other_data, dimensions
+        dims = self.dims
+    return self_data, other_data, dims
